/*____________________________________________________________________________*\
 *

 Copyright (c) 1997 John Roy. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
    mentioning features must contain the following
    acknowledgment:
    "This product includes software developed by John Roy
    (http://www.johnroy.com/pi3/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: MiscUtil.cpp$
 * $Date: Sun Aug 10 06:28:00 1997$
 *
 Description:
\*____________________________________________________________________________*/
//$SourceTop:$

#include <assert.h>
#include "MiscUtil.h"

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
	If the relative path begins with '/' or '\' or
	is of the form 'D:' where D is any single letter, then the 
	path is assumed to be an absolute path.
\*___________________________________________________________________________*/
bool IsAbsolutePath( const char *pPath )
{
	assert( pPath );
	if ( !pPath )
		{ return false; };
	char c = *pPath;
	if ( !c )
		{ return false; };
	if ( c=='/'	|| c=='\\' )
		{ return true; };
	c = pPath[1];
	if ( c==':' ) { return true; };
	return false;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
	If the relative path appears to be absolute path, as defined by the
	above function, then it is returned intact.
	Assumes root path is a correctly formatted path with no '.' or '..'
	components.

	Only handles simple relative paths like './file' and
	'../.././file', not '../adaf/../file' or '/////file'.
\*____________________________________________________________________________*/
bool RelativeToAbsolutePath(
	const char *pRoot,
	const char *pRelPath,
	PIString &sAbsolutePath )
{
	assert( pRoot && pRelPath ); if ( !pRoot || !pRelPath ) { return false; };
	if ( IsAbsolutePath( pRelPath ) )
		{ sAbsolutePath = pRelPath;
		return true; };

	/* --- get length of absolute path --- */
	int iRootLen = strlen( pRoot );
	if ( iRootLen==0 )
		{ sAbsolutePath = pRelPath;
		return true; };

	/* --- ignore trailing slash on absolute path --- */
	if ( pRoot[iRootLen-1]=='/' || pRoot[iRootLen-1]=='\\' )
		{ iRootLen--; };

	/* --- do some simple contraction --- */
	for(;;)
		{
		if (	!strncmp( pRelPath, "./", 2 ) ||
				!strncmp( pRelPath, ".\\", 2 ) )
			{
			pRelPath = &( pRelPath[2] );
			continue;
			}
		else if (	!strncmp( pRelPath, "../", 3 ) ||
				!strncmp( pRelPath, "..\\", 3 ) )
			{
			pRelPath = &( pRelPath[3] );
			int i=iRootLen-1;
			for( ; i>=0; i--)
				{
				char c = pRoot[i];
				if ( c=='/' || c=='\\' )
					{ break; };
				};
			iRootLen = i;
			continue;
			}
		else
			{ break; };
		};

	PIString sTmp( pRoot, iRootLen );
	sTmp.Concatenate( '/' );
	sTmp.Concatenate( pRelPath );

	sAbsolutePath = sTmp;

	return true;
}

