/*____________________________________________________________________________*\
 *

 Copyright (c) 1997 John Roy. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
    mentioning features must contain the following
    acknowledgment:
    "This product includes software developed by John Roy
    (http://www.johnroy.com/pi3/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: PIString.cpp$
 * $Date: Sun Aug 10 06:28:02 1997$
 *
 Description:
\*____________________________________________________________________________*/
//$SourceTop:$

#include <assert.h>
#include <string.h>
#include <ctype.h>

#include "PIString.h"
#include "PIAlloc.h"

/*____________________________________________________________________________*\
 *
 Global declarations:
\*____________________________________________________________________________*/
PIString PIString::sEmpty;

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PIString::PIString()
:	pStr(0), iLen(0)
{
	NewString(0, 0);
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PIString::PIString(const char *pS, int iTheLen)
:	pStr(0), iLen(0)
{
	NewString(pS, iTheLen);
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PIString::PIString(const PIString &s)
:	pStr(0), iLen(0)
{
	NewString( s.pStr, s.iLen );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
void PIString::Concatenate(const PIString &s)
{
	/* --- other string is empty --- */
	if ( !s.pStr || !s.iLen)
		{ assert( !s.iLen ); return; };

	/* --- this string is empty --- */
	if ( !iLen )
		{
		NewString( s.pStr, s.iLen );
		return;
		};

	/* --- where both strings fit within this one --- */
	if ( (iLen+s.iLen)<BUF_SIZE )
		{
		assert( pStr && s.pStr );
		strcat( pStr, s.pStr );
		}
	else
		{
		/* --- otherwise most difficult case, new allocation --- */
		char *pTmp = PI_NEW( char[iLen+s.iLen+1] );
		assert( pTmp );
		assert( pStr );
		assert( pTmp );
		strcpy( pTmp, pStr );
		if ( iLen>=BUF_SIZE )
			{ PI_DELETE( pStr ); };
		strcat( pTmp, s.pStr );
		pStr = pTmp;
		};

	iLen += s.iLen;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
	This could be made a lot better.
\*____________________________________________________________________________*/
void PIString::Concatenate(const char c)
{
	char szBuf[2];
	szBuf[0]=c;
	szBuf[1]='\0';
	Concatenate(PIString(szBuf));
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PIString::~PIString()
{
	if ( iLen>=BUF_SIZE )
		{ PI_DELETE( [] pStr ); };
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
	Note this fn. is required to put iLen into a deterministic state.
\*____________________________________________________________________________*/
void PIString::NewString(const char *pS, int iTheLen )
{
	if ( pS==pStr )
		{ return; };

	if ( iLen>=BUF_SIZE )
		{ PI_DELETE( pStr ); };
		
	if ( pS!=0 )
		{
		if ( iTheLen<0 ) 
			{ iLen=strlen(pS); }
		else
			{ iLen=iTheLen; };

		if ( iLen<BUF_SIZE /* same thing as (iLen+1)<=BUF_SIZE */ )
			{ pStr = szBuf; }
		else
			{ pStr = PI_NEW(char[iLen+1] ); };

		strncpy( pStr, pS, iLen );
		pStr[iLen]='\0';
		}
	else
		{
		pStr=0;
		iLen=0;
		};
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PIString::operator const char*() const
{
	return pStr?pStr:"";
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
const int PIString::operator==(const PIString &s) const
{
	if ( iLen!=s.iLen )
		{ return 0; };
	if ( !iLen )
		{ return 1; };
	assert( pStr && s.pStr );
	return !strcmp( pStr, s.pStr );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
const int PIString::operator==(const char *pS ) const
{
	return !strcmp( pStr?pStr:"", pS?pS:"" );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PIString &PIString::operator=(const PIString &s)
{
	if ( pStr!=s.pStr )
		NewString( s.pStr, -1 );
	return *this;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
void PIString::ConvertToUpperCase()
{
	if ( pStr )
		{ 
		for(int i=0; i<Len(); i++)
			{
			pStr[i]=(char)toupper( pStr[i] );
			};
		};
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
void PIString::Clear()
{
	if ( iLen>=BUF_SIZE )
		{ PI_DELETE( [] pStr ); };
	pStr=0;
	iLen=0;
}

#if 0
/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
char *PIString::NewCharArray() const
{
	char *pNew = PI_NEW( char[ pStr&&iLen ? iLen : 0 ] );
	if ( pStr && iLen )
		{ strcpy( pNew, pStr ); }
	else
		{ *pStr = '\0'; };
	return pNew;
}
#endif

