/*____________________________________________________________________________*\
 *

 Copyright (c) 1997 John Roy. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
    mentioning features must contain the following
    acknowledgment:
    "This product includes software developed by John Roy
    (http://www.johnroy.com/pi3/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: PIString.h$
 * $Date: Sun Aug 10 06:28:02 1997$
 *
 Description:
\*____________________________________________________________________________*/
//$HeaderTop:$

#ifndef PISTRING_H_
#define PISTRING_H_

#include <iostream.h>
#include <string.h>

/*____________________________________________________________________________*\
 *
 Class:
 Description:
\*____________________________________________________________________________*/
class PIString
{
private:
	/* ---
	this internal buffer is for optimization purposes. Allocations
	<=BUF_SIZE won't be allocated off the heap, instead this internal	
	buffer will be used, this can be disabled by setting BUF_SIZE to 0 (
	or 1).
	--- */
	enum { BUF_SIZE=256 };
	char szBuf[BUF_SIZE];

	char *pStr;
	int iLen;
	static PIString sEmpty;

	/* --- internal methods --- */
	void NewString(const char *pS, int iLen);

public:
	PIString();
	PIString(const char *pS, int iLen=-1);
	PIString(const PIString &s);
	void Concatenate(const PIString &s);
	void Concatenate(const char c);
	~PIString();
	operator const char*() const;
	const int operator==(const PIString &s) const;
	const int operator==(const char *pS ) const;
	inline const int operator!=(const PIString &s) const
		{ return !((*this)==s); };
	PIString &operator=(const PIString &s);
	inline int Len() const	{ return iLen; };
	void ConvertToUpperCase();
	void Clear();
//	char *NewCharArray() const;

	inline static const PIString &Empty()	{ return sEmpty; };
};

#endif	/* PISTRING_H_ */

