/*____________________________________________________________________________*\
 *

 Copyright (c) 1997 John Roy. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
	notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
	notice, this list of conditions and the following disclaimer in
	the documentation and/or other materials provided with the
	distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
	mentioning features must contain the following
	acknowledgment:
	"This product includes software developed by John Roy
	(http://www.johnroy.com/pi3/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: ISAPI20.cpp$
 * $Date: Sun Aug 10 06:36:50 1997$
 *
 Description:
	Execute an ISAPI plugin.

\*____________________________________________________________________________*/
//$SourceTop:$

#include <iostream.h>
#include <stdio.h>

#if WIN32
#include <HTTPExt.h>
#endif


#include "HandBase.h"
#include "HTTPCore.h"
#include "HTTPUtil.h"
#include "PIStrStr.h"

#if WIN32

/*
** #define D { cerr << __FILE__ << ": " << __LINE__ << endl; }
*/
#define D

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
#if 0
	/*
	** HTML documentation for this handler
	*/
/*___+++HTMLDOC_BEGIN+++___*/
Name:
	ISAPI20

Description:
	Load and execute an ISAPI (isa) extension.

<!--

Old comments

<TR>
<TD>HeaderPattern
<TD>-
<TD>A Pi3Expression
<TD>Output first
<TD>HeaderPattern="<HTML><TITLE>The Title</TITLE>"

<TR>
<TD>FooterPattern
<TD>-
<TD>A Pi3Expression
<TD>Output last
<TD>FooterPattern="</HTML>"

<TR>
</TABLE>
<STRONG>-</STRONG> in the <IT>default</IT> indicates no default<BR>
<STRONG>+</STRONG> in the <IT>default</IT> indicates the field is mandatory<BR>

<H4>Description of Options</H4>

<H5>
	HeaderPattern
</H5>
Specifies a pattern of data that will be pre-appended to the file.

<H5>
	FooterPattern
</H5>
Specifies a pattern of data that will be appended to the file.
-->

Phase:
	HANDLE

Returns:
	PIAPI_COMPLETED, PIAPI_CONTINUE, PIAPI_ERROR, or INT_REDIRECT according
	to the status returned by the extension.

Note:
Example:
	<PRE>
	&ltObject&gt
		Name ISAPI20
		Class ISAPI20Class
	&lt/Object&gt

	&ltObject&gt
		...
		Handle Condition="&cmp(&dblookup(response,string,ObjectMap,ISAPI),ISAPI)" \
			ISAPI20
		...
	&lt/Object&gt

	</PRE>

/*___+++HTMLDOC_END+++___*/
#endif

/*____________________________________________________________________________*\
 *
 Class:
 Description:
	Prototypes for callback functions
	Class for ISAPI context
\*____________________________________________________________________________*/
BOOL WINAPI fnGetServerVariable( HCONN	   hConn,
					LPSTR	   lpszVariableName,
					LPVOID	  lpvBuffer,
					LPDWORD	 lpdwSize );

BOOL WINAPI fnWriteClient( HCONN	  ConnID,
					LPVOID	 Buffer,
					LPDWORD	lpdwBytes,
					DWORD	  dwReserved );

BOOL WINAPI fnReadClient( HCONN	  hConn,
					LPVOID	 lpvBuffer,
					LPDWORD	lpdwSize );

BOOL WINAPI fnServerSupportFunction( HCONN	  hConn,
					DWORD	  dwHSERRequest,
					LPVOID	 lpvBuffer,
					LPDWORD	lpdwSize,
					LPDWORD	lpdwDataType );


/*____________________________________________________________________________*\
 *
 Class:
 Description:
	Class for ISAPI context
\*____________________________________________________________________________*/
class ISAPI20;
class ISAPIContext : public _EXTENSION_CONTROL_BLOCK
{
public:
	ISAPI20 *pISAPI20;
	PIHTTP *pPIHTTP;
	int iRc;

private:
	inline char *DBL( PIDB *pDB, int iType, const char *pKey )
		{
		char *pTmp = (char *)PIDB_lookup( pDB, iType, pKey, 0 );
		return pTmp ? pTmp : "";
		};

public:
	ISAPIContext( ISAPI20 *pTheISAPI20, PIHTTP *pThePIHTTP )
	:	pISAPI20( pTheISAPI20 ), pPIHTTP( pThePIHTTP ), iRc( PIAPI_COMPLETED )
		{
		/* --- locals --- */
		PIDB *pQ = pPIHTTP->pRequestDB;
		PIDB *pR = pPIHTTP->pResponseDB;

		/* --- initialize member data --- */
		cbSize = sizeof( _EXTENSION_CONTROL_BLOCK );
		dwVersion = MAKELONG( HSE_VERSION_MINOR, HSE_VERSION_MAJOR );
		ConnID = (HCONN)this;
		dwHttpStatusCode = 0;
		*lpszLogData = '\0';

		/* --- input variables --- */
		lpszMethod = DBL( pQ, PIDBTYPE_STRING, KEY_HTTP_METHOD );
		lpszQueryString = DBL( pQ, PIDBTYPE_STRING, KEY_HTTP_QUERYSTRING );
		lpszPathInfo = DBL( pR, PIDBTYPE_STRING, KEY_INT_PATHINFO );
		lpszPathTranslated = DBL(pR, PIDBTYPE_STRING, KEY_INT_PATHTRANSLATED);
		lpszContentType = DBL( pQ, PIDBTYPE_RFC822, KEY_HTTP_CONTENTTYPE );


		/* --- check for stdin --- */
		cbAvailable = 0;
		lpbData = NULL;
		cbTotalBytes = 0;
		const char *pContentLength = DBL( pQ, PIDBTYPE_RFC822,
			KEY_HTTP_CONTENTLENGTH );
		cbTotalBytes = pContentLength ? atol( pContentLength ) : 0;

		/* --- callback functions --- */
		GetServerVariable = fnGetServerVariable;
		WriteClient = fnWriteClient;
		ReadClient = fnReadClient;
		ServerSupportFunction = fnServerSupportFunction;
		};
};

/*____________________________________________________________________________*\
 *
 Class:
 Description:
\*____________________________________________________________________________*/
class ISAPI20 : public HandlerBaseISAPI
{
private:
	/* ---
	Configuration data
	--- */
	PIDB *pTypes;					/* DB for variable types */

protected:
	/* ---
	Lookup functions (for GetServerVariable)
	--- */
	typedef BOOL (* PMFN)( ISAPI20 *pThis, PIHTTP *, LPVOID, LPDWORD );
# define FN_DB_STR( name, db, type, key ) \
	static BOOL (name)( ISAPI20 *, PIHTTP *pPIHTTP, LPVOID lpvBuffer, \
			LPDWORD lpdwSize )											\
		{																\
		const char *pValue = (const char *)PIDB_lookup( pPIHTTP->db,	\
			type, key, 0 );												\
		if ( !pValue )													\
			{															\
			pValue = ""; 												\
			};															\
		int iToCopy = strlen( pValue );									\
		*lpdwSize = iToCopy;											\
		if ( iToCopy>*lpdwSize || !lpvBuffer )							\
			{															\
			::SetLastError( ERROR_INSUFFICIENT_BUFFER );				\
			return FALSE;												\
			};															\
		strncpy( (char *)lpvBuffer, pValue, iToCopy );					\
		return TRUE;													\
		};			

# define FN_STRING( name, value ) \
	static BOOL (name)( ISAPI20 *, PIHTTP *pPIHTTP, LPVOID lpvBuffer, \
			LPDWORD lpdwSize )											\
		{																\
		const char *pValue = (value);									\
		if ( !pValue )													\
			{															\
			pValue = ""; 												\
			};															\
		int iToCopy = strlen( pValue );									\
		*lpdwSize = iToCopy;											\
		if ( iToCopy>*lpdwSize || !lpvBuffer )							\
			{															\
			*lpdwSize = iToCopy;										\
			::SetLastError( ERROR_INSUFFICIENT_BUFFER );				\
			return FALSE;												\
			};															\
		strncpy( (char *)lpvBuffer, pValue, iToCopy );					\
		return TRUE;													\
		};			

# define FN_DB_NUM( name, db, type, key ) \
	static BOOL (name)( ISAPI20 *, PIHTTP *pPIHTTP, LPVOID lpvBuffer,	\
			LPDWORD lpdwSize )											\
		{																\
		*lpdwSize = sizeof(DWORD);										\
		if ( *lpdwSize<sizeof(DWORD) )									\
			{															\
			::SetLastError( ERROR_INSUFFICIENT_BUFFER );				\
			return FALSE;												\
			};															\
		const char *pValue = (const char *)PIDB_lookup( pPIHTTP->db,	\
			type, key, 0 );												\
		*((DWORD *)lpvBuffer ) = pValue ? atol( pValue ) : 0; 			\
		return TRUE;													\
		};			

	FN_DB_STR( fnAuthType, pResponseDB, PIDBTYPE_STRING, KEY_INT_AUTHTYPE );
	FN_DB_NUM( fnContentLength, pRequestDB, PIDBTYPE_RFC822,
		KEY_HTTP_CONTENTLENGTH );
	FN_DB_STR( fnContentType, pRequestDB, PIDBTYPE_RFC822,KEY_HTTP_CONTENTTYPE);

	/*
	** And why does this ISAPI module care about what version of CGI another
	** part of the server implements???, encapsulation my dear Watson....
	*/
	FN_STRING( fnGatewayInterface, "CGI/1.1" );
	FN_DB_STR( fnPathInfo, pResponseDB, PIDBTYPE_STRING, KEY_INT_PATHINFO );
	FN_DB_STR( fnPathTranslated, pResponseDB, PIDBTYPE_STRING,
		KEY_INT_PATHTRANSLATED );
	FN_DB_STR( fnQueryString, pRequestDB, PIDBTYPE_STRING,KEY_HTTP_QUERYSTRING);
	FN_DB_STR( fnRemoteAddr, pConnectionDB, PIDBTYPE_STRING,KEY_INT_REMOTEADDR);
	FN_DB_STR( fnRemoteHost, pConnectionDB, PIDBTYPE_STRING,KEY_INT_REMOTEHOST);
	FN_DB_STR( fnRemoteUser, pResponseDB, PIDBTYPE_STRING,KEY_INT_REMOTEUSER );
	FN_DB_STR( fnRequestMethod, pRequestDB, PIDBTYPE_STRING, KEY_HTTP_METHOD );
	FN_DB_STR( fnScriptName, pResponseDB, PIDBTYPE_STRING, KEY_INT_SCRIPTNAME );
	FN_DB_STR( fnServerName, pHostDB, PIDBTYPE_RFC822, KEY_INT_HOSTNAME );
	FN_DB_STR( fnServerPort, pHostDB, PIDBTYPE_RFC822, KEY_INT_SERVERPORT );
	FN_DB_STR( fnServerProtocol,pRequestDB,PIDBTYPE_STRING,KEY_HTTP_PROTOCOL );
	FN_STRING( fnServerSoftware, HTTPCore_getServerStamp() );
	FN_DB_STR( fnAuthPass, pResponseDB, PIDBTYPE_STRING, KEY_INT_AUTHPASS );
#undef FN_DB_STR

	int Parameter( const char *pVariable, const char *pValue,
		const char *pWhere )
		{
		assert( pVariable && pValue );
		PIOStrStream os;
		os << pWhere << "ISAPI20: ";
		if ( 0 )
			{
			}
		else
			{
			os << "Unknown directive '" << pVariable <<
				"'" << ends;
			CONFIG_ERR( Object(), os.str() );
			return 0;
			};

		return 1;
		};

public:
	ISAPI20( PIObject *pObject, int iArgc, const char *ppArgv[] )
	:	HandlerBaseISAPI( pObject ),
		pTypes( PIDB_new( 0, "Types" ) )
		{
		ReadParameters( iArgc, ppArgv );

		/* ---
		Insert DB types for 'GetServerVariable'
		--- */
#define AT( name, function ) \
		PIDB_add( pTypes, PIDBTYPE_OPAQUE, (name), (void *)function, 0 );

		AT( "AUTH_TYPE", fnAuthType );
		AT( "CONTENT_LENGTH", fnContentLength );
		AT( "CONTENT_TYPE", fnContentType );
		AT( "GATEWAY_INTERFACE", fnGatewayInterface );
		AT( "PATH_INFO", fnPathInfo );
		AT( "PATH_TRANSLATED", fnPathTranslated );
		AT( "QUERY_STRING", fnQueryString );
		AT( "REMOTE_ADDR", fnRemoteAddr );
		AT( "REMOTE_HOST", fnRemoteHost );
		AT( "REMOTE_USER", fnRemoteUser );
		AT( "REQUEST_METHOD", fnRequestMethod );
		AT( "SCRIPT_NAME", fnScriptName );
		AT( "SERVER_NAME", fnServerName );
		AT( "SERVER_PORT", fnServerPort );
		AT( "SERVER_PROTOCOL", fnServerProtocol );
		AT( "SERVER_SOFTWARE", fnServerSoftware );
		AT( "AUTH_PASS", fnAuthPass );
#undef AT
		};

	~ISAPI20()
		{
		PIDB_delete( pTypes );
		};

	/* ---
	The real stuff
	--- */
	int RunISAPI( PIHTTP *pPIHTTP, PFN_HTTPEXTENSIONPROC fnHttpExtensionProc,
		const char *pPath )
		{
		assert( pPIHTTP );

		/* ---
		Create new ISAPI context
		--- */
		ISAPIContext tContext( this, pPIHTTP );

		/* ---
		Read in stdin (if any)
		--- */
		if ( tContext.cbTotalBytes )
			{
			/* ---
			See how much we can read on the first pass and leave the	
			extension for call ReadClient() for the rest		
			--- */
			int iRead = 0;
			const char *pBuffer = PIIOBuffer_read( pPIHTTP->pBuffer,
				&iRead );
			if ( !pBuffer || !iRead )
				{
				/* ---
				This is an error
				--- */
				HTTPCore_logError( pPIHTTP, "ISAPI20: Could not read stdin \
(%ld bytes) for ISAPI DLL '%s'.", (long)tContext.cbTotalBytes, pPath );
				return HTTPUtil_doHTTPError( pPIHTTP, ST_INTERNALERROR );
				};
			
			/* --- 
			Allocate a buffer and put data in it, buffer size will be
			no more that internal buffer in PIIOBuffer object.
			(usually between 4 and 16 K).
			--- */
			tContext.lpbData = (LPBYTE)PIHTTP_allocMem( pPIHTTP, iRead );
			memcpy( tContext.lpbData, pBuffer, iRead );
			tContext.cbAvailable = iRead;
			};

		/* ---
		Fire off extension
		--- */
		int iRet = (fnHttpExtensionProc)( &tContext );

		/* ---
		Check for error message
		--- */
		if ( tContext.lpszLogData && *(tContext.lpszLogData) )
			{
			HTTPCore_logError( pPIHTTP, "ISAPI20: ISAPI dll with path '%s' \
logged an error message: ________________________________________________________________________________\n%s\n________________________________________________________________________________", pPath, tContext.lpszLogData );
			};
		
		pPIHTTP->iStatus = tContext.dwHttpStatusCode;

		switch( iRet )
			{
			case HSE_STATUS_SUCCESS:
				/*
				** Assume Content-Length could not be set, disable
				** keep open
				*/
				PIDB_replace( pPIHTTP->pConnectionDB, PIDBTYPE_OPAQUE,
					KEY_INT_KEEPOPEN, 0, 0 );
				break;

			case HSE_STATUS_SUCCESS_AND_KEEP_CONN:
				/*
				** OK
				*/
				break;

			case HSE_STATUS_PENDING:
				/*
				** Not supported
				*/
				HTTPCore_logError( pPIHTTP, "ISAPI20: ISAPI extension \
with path '%s' returned HSE_STATUS_PENDING. This is not implemented", pPath );
				return HTTPUtil_doHTTPError( pPIHTTP, ST_NOTIMPLEMENTED );

			case HSE_STATUS_ERROR:
			default:
				HTTPCore_logError( pPIHTTP, "ISAPI20: ISAPI extension \
with path '%s' returned HSE_STATUS_ERROR or an unknown return code.", pPath );
/*
** Don't send this because the plugin should have sent some headers
*/
/*				return HTTPUtil_doHTTPError( pPIHTTP, ST_INTERNALERROR ); */

			};

		return tContext.iRc;
		};

	/* ---
	The real stuff
	--- */
	int DoISAPI( PIDLL *pDLL, const char *pPath, PIHTTP *pPIHTTP )
		{
		assert( pDLL );

		PFN_GETEXTENSIONVERSION fnGetVersion = (PFN_GETEXTENSIONVERSION)
			PIDLL_getAddress( pDLL, "GetExtensionVersion" );
		if ( !fnGetVersion )
			{
			HTTPCore_logError( pPIHTTP, "ISAPI20: Could not load function \
'GetExtensionVersion' from ISAPI DLL '%s'.", pPath );
			return HTTPUtil_doHTTPError( pPIHTTP, ST_INTERNALERROR );
			};


		PFN_HTTPEXTENSIONPROC fnHttpExtensionProc = (PFN_HTTPEXTENSIONPROC)
			PIDLL_getAddress( pDLL, "HttpExtensionProc" );
		if ( !fnHttpExtensionProc )
			{
			HTTPCore_logError( pPIHTTP, "ISAPI20: Could not load function \
'HttpExtensionProc' from ISAPI DLL '%s'.", pPath );
			return HTTPUtil_doHTTPError( pPIHTTP, ST_INTERNALERROR );
			};

		PFN_TERMINATEEXTENSION fnTerminateExtension = (PFN_TERMINATEEXTENSION)
			PIDLL_getAddress( pDLL, "TerminateExtension" );

		/* ---
		No check, fnTerminateExtension is optional
		--- */

		/* ---
		Do GetExtensionVersion
		--- */
		HSE_VERSION_INFO tInfo;
		if ( !( (fnGetVersion)( &tInfo ) ) )
			{
			HTTPCore_logError( pPIHTTP, "ISAPI20: Call to \
'GetExtensionVersion' failed for ISAPI DLL '%s'. Win32 Error code is %d.",
				pPath , ::GetLastError() );
			return HTTPUtil_doHTTPError( pPIHTTP, ST_INTERNALERROR );
			};

		/* ---
		OK do the DLL processing
		--- */
		int iRet = RunISAPI( pPIHTTP, fnHttpExtensionProc, pPath );
	
		/* ---
		Unload DLL
		--- */
		if ( fnTerminateExtension )
			{
			(fnTerminateExtension)(HSE_TERM_MUST_UNLOAD);
			};
		
		return iRet;
		};

	int Handle( int iPhase, PIHTTP &tPIHTTP, PIIOBuffer &/* tBuffer */ )
		{
		if ( iPhase!=PH_HANDLE )
			{ return PIAPI_ERROR; };

		PIDB *pR = tPIHTTP.pResponseDB;
		PIDB *pQ = tPIHTTP.pRequestDB;

		/* ---
		Get file object
		--- */
		const char *pPath = (const char *)PIDB_lookup( pR, PIDBTYPE_STRING,
			KEY_INT_PATH, 0 );
		PIFInfo *pFInfo = HTTPCore_getCachedFile( pPath ); 
		if ( !pFInfo )
			{ return PIAPI_ERROR; };

		/* ---	
		This file must exist and be a regular file 
		--- */
		if ( !PIFInfo_exists( pFInfo ) ||
			!PIFInfo_isRegular( pFInfo ) )
			{
			HTTPCore_logError( &tPIHTTP, "ISAPI20: File with \
path '%s' does not \
exist or is not a regular file.", pPath );
			HTTPCore_releaseCachedFile( pFInfo );	
			return HTTPUtil_doHTTPError( &tPIHTTP, ST_INTERNALERROR );
			};

		PIDLL *pDLL = PIDLL_new( pPath );
		if ( !PIDLL_isLoaded( pDLL ) )
			{
			const char *pDesc = PIDLL_getErrorDescription( pDLL );
			HTTPCore_logError( &tPIHTTP, "ISAPI20: ISAPI DLL with \
path '%s' could not be loaded, error description is \
'%s'.", pPath, pDesc );
			HTTPCore_releaseCachedFile( pFInfo );	
			return HTTPUtil_doHTTPError( &tPIHTTP, ST_INTERNALERROR );
			};

		int iRet = DoISAPI( pDLL, pPath, &tPIHTTP );
		PIDLL_delete( pDLL );

		HTTPCore_releaseCachedFile( pFInfo );

		return iRet;
		};

	BOOL GetServerVariable( PIHTTP *pPIHTTP, LPSTR lpszVariableName,
		LPVOID lpvBuffer, LPDWORD lpdwSize )
		{
		if ( !lpszVariableName || !lpdwSize )
			{
			::SetLastError( ERROR_INVALID_PARAMETER );
			return FALSE;
			};

		/* ---
		Convert variable to upper case
		--- */
		PIString sTmp( lpszVariableName );
		sTmp.ConvertToUpperCase();

		/* ---
		Lookup function corresponding to this environment variable
		--- */
		PMFN fnEnv = (PMFN)PIDB_lookup( pTypes, PIDBTYPE_OPAQUE, sTmp, 0 );

		/* ---
		No lookup function?
		--- */
		if ( !fnEnv )
			{
			::SetLastError( ERROR_INVALID_INDEX );
			return FALSE;
			};

		return (fnEnv)( this, pPIHTTP, lpvBuffer, lpdwSize );
		};
};

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
BOOL WINAPI fnGetServerVariable( HCONN hConn, LPSTR lpszVariableName,
	LPVOID lpvBuffer, LPDWORD lpdwSize )
{
	ISAPIContext *pContext = (ISAPIContext *)hConn;
	int iRet = pContext->pISAPI20->GetServerVariable( pContext->pPIHTTP,
		lpszVariableName, lpvBuffer, lpdwSize );
	if ( HTTPCore_debugEnabled() )
		{
		HTTPCore_logDebug( DBG_MED, "ISAPI20: Return code<%d>, variable<%s>, \
value<%x>, size<%d>.",
			iRet, lpszVariableName ? lpszVariableName : "",
			(void *)lpvBuffer, lpdwSize ? *lpdwSize : 0 );
		};
	return iRet;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
BOOL WINAPI fnWriteClient( HCONN ConnID, LPVOID Buffer, LPDWORD	lpdwBytes,
	DWORD dwReserved )
{
	if ( !ConnID || !lpdwBytes || !Buffer )
		{
		::SetLastError( ERROR_INVALID_PARAMETER );
		return FALSE;
		};

	ISAPIContext *pContext = (ISAPIContext *)ConnID;
	int iWritten = PIIOBuffer_write( pContext->pPIHTTP->pBuffer,
		(char *)Buffer, *lpdwBytes, PIIOBUF_NONE );
	if ( iWritten==-1 )
		{
		/*
		** Socket must have been closed
		*/
		::SetLastError( ERROR_INVALID_HANDLE );
		return FALSE;
		};
	*lpdwBytes = iWritten;
	return TRUE;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
BOOL WINAPI fnReadClient( HCONN ConnID, LPVOID lpvBuffer, LPDWORD lpdwSize )
{
	if ( !ConnID || !lpdwSize )
		{
		::SetLastError( ERROR_INVALID_PARAMETER );
		return FALSE;
		};

	ISAPIContext *pContext = (ISAPIContext *)ConnID;
	int iRet = PIIOBuffer_readToBuffer( pContext->pPIHTTP->pBuffer,
		(char *)lpvBuffer, *lpdwSize );
	if ( iRet>0 )
		{
		*lpdwSize = iRet;
		return TRUE;
		}
	else
		{
		return FALSE;
		};
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
BOOL WINAPI fnServerSupportFunction( HCONN ConnID, DWORD dwHSERRequest,
	LPVOID lpvBuffer, LPDWORD lpdwSize, LPDWORD	lpdwDataType )
{
	if ( !ConnID )
		{
		::SetLastError( ERROR_INVALID_PARAMETER );
		return FALSE;
		};

	ISAPIContext *pContext = (ISAPIContext *)ConnID;
	PIHTTP *pPIHTTP = pContext->pPIHTTP;

	switch( dwHSERRequest )
		{
		case HSE_REQ_SEND_URL_REDIRECT_RESP:
			PIDB_replace( pPIHTTP->pResponseDB, PIDBTYPE_RFC822,
				KEY_HTTP_LOCATION, lpvBuffer, 0 );
			pContext->dwHttpStatusCode = ST_PERMANENTREDIRECT;
			pContext->iRc = PIAPI_COMPLETED;
			return TRUE;

		case HSE_REQ_SEND_URL:
			pContext->iRc = HTTPUtil_doHTTPError(pPIHTTP,ST_PERMANENTREDIRECT);
			return TRUE;

		case HSE_REQ_SEND_RESPONSE_HEADER:
			{
			int iStatus = lpvBuffer ? atoi( (char *)lpvBuffer ) : 200;	
			pPIHTTP->iStatus = iStatus;
			pContext->dwHttpStatusCode = iStatus;
			pContext->iRc = PIAPI_COMPLETED;

			/*
			** Send server specific headers
			*/
			HTTPCore_sendGeneralHeaders( pPIHTTP );
			if ( lpdwSize && *lpdwSize && lpdwDataType )
				{
				PIIOBuffer_write( pPIHTTP->pBuffer, (const char *)lpdwDataType,
					*lpdwSize, PIIOBUF_NONE );
				};
			};
			return TRUE;
		case HSE_REQ_DONE_WITH_SESSION:
		default:;
		};
	return FALSE;
}

#endif /* WIN32 */

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int ISAPI20_constructor( PIObject *pObj,
	int iArgc, const char *ppArgv[] )
{
#if WIN32
	return HandlerBaseISAPI_constructor( pObj, PI_NEW( ISAPI20( pObj,
		iArgc, ppArgv ) ) );
#elif POSIX
	(void)pObj;
	(void)iArgc;
	(void)ppArgv;
	return PIAPI_COMPLETED;
#endif
}

#if 0
/*___+++CNF_BEGIN+++___*/
	<Class>
		Name ISAPI20Class
		Type LogicExtension
		Library ISAPI
		OnClassLoad HandlerBaseISAPI_onClassLoad
		Constructor ISAPI20_constructor
		CopyConstructor HandlerBaseISAPI_copyConstructor
		Destructor HandlerBaseISAPI_destructor
		Execute HandlerBaseISAPI_execute
	</Class>

	<Object>
		Name ISAPI20
		Class ISAPI20Class
	</Object>

/*___+++CNF_END+++___*/
#endif

