/*____________________________________________________________________________*\
 *

 Copyright (c) 1997 John Roy. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
    mentioning features must contain the following
    acknowledgment:
    "This product includes software developed by John Roy
    (http://www.johnroy.com/pi3/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: Interface.h$
 * $Date: Sun Aug 10 06:36:32 1997$
 *
 Description:

\*____________________________________________________________________________*/
/* $SourceTop:$ */

#ifndef INTRFACE_H_
#define INTRFACE_H_

#if WIN32
#include <windows.h>
#endif

#include "PIString.h"
#include "Pi2API.h"

#include "IFaceDef.h"

	/* -------------- +++++++++++++++++++++++++++++++ ------------------ *\
	
						Internal C++ Interface

	\* -------------- +++++++++++++++++++++++++++++++ ------------------ */
#if defined(__cplusplus) && defined(PI3_INTERNAL)

#include <assert.h>
#include <stddef.h>

#include "DblList.h"		/* --- from Base library --- */

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
class Interface
{
private:
	/* --- copy constructor will be disallowed --- */
	Interface( const Interface &tH );

public:

#if WIN32

    /* Windows Specific Stuff */
    /* 
    ** Information on window
    */
    PIString sError;
//    HWND hWnd; --- LATER
    HCURSOR hHourglass;
    HCURSOR hSave;

#endif
    int iMessage;

	Interface()
    :   
#if WIN32
        hHourglass( ::LoadCursor( 0, MAKEINTRESOURCE( IDC_WAIT ) ) ),
        hSave( 0 )
#endif
        {};

	~Interface()
        {
        ::FreeResource( hHourglass );
        };

    void HourglassOn()
        {
        assert( !hSave );
        if ( hSave ) { return; };
        hSave = ::SetCursor( hHourglass );
        };

    void HourglassOff()
        {
        assert( hSave );
        if ( !hSave ) { return; };
        ::SetCursor( hSave );
        hSave = 0;
        };
};

	/* -------------- +++++++++++++++++++++++++++++++ ------------------ *\
	
						External C Interface

	\* -------------- +++++++++++++++++++++++++++++++ ------------------ */
#else
/*____________________________________________________________________________*\
 *
 Description:
	C structure version of C++ class.
\*____________________________________________________________________________*/
struct _Interface
{
};
typedef struct _Interface Interface;

#endif
	/* -------------- +++++++++++++++++++++++++++++++ ------------------ *\
	
						C Functions

	\* -------------- +++++++++++++++++++++++++++++++ ------------------ */
/*____________________________________________________________________________*\
 *
 Name:
	Interface_new

 Synopsis:
	Interface *Interface_new()

 Description:
	This function is not yet documented.

 Notes:
 Return Values:
 Errors:
 See Also:
\*____________________________________________________________________________*/
PUBLIC_PIAPI Interface *Interface_new();

/*____________________________________________________________________________*\
 *
 Name:
	Interface_delete

 Synopsis:
	int Interface_delete( Interface *pInterface )

 Description:
	This function is not yet documented.

 Notes:
 Return Values:
 Errors:
 See Also:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int Interface_delete( Interface *pInterface );

#endif /* INTRFACE_H_ */

