/*____________________________________________________________________________*\
 *

 Copyright (c) 1997 John Roy. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
    mentioning features must contain the following
    acknowledgment:
    "This product includes software developed by John Roy
    (http://www.johnroy.com/pi3/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: Pi3Win.c$
 * $Date: Sun Aug 10 06:40:13 1997$
 *
 Description:

\*____________________________________________________________________________*/
/* $SourceTop:$ */

#include <assert.h>
#include <ostream.h>
#include <stdlib.h>
#include <ctype.h>
#include <fstream.h>

#include "PiString.h"
#include "Pi3File.h"

/*
** #define D { cerr << __FILE__ << ": " << __LINE__ << endl; }
*/
#define D

#define SECTION_START "+++---AD Section Start, Do not Modify:"
#define SECTION_END "+++---AD Section End---+++"
#define NOTMODIFIED_START "+++---AD Saved Section, modifications will be retained---+++"
#define NOTMODIFIED_END "+++---AD Saved section end---+++"
#define GENERATED_START "+++---AD Generated Section, modifications will be lost---+++"
#define GENERATED_END "+++---AD Generated section end---+++"

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
void Internal_setupDefaults( QConfig *pConfig )
{
#define AD_DEFAULT(x,y,z) \
	{ \
	if ( !M_LookupValue( pConfig, x, y ) ) \
		{ M_ReplaceValue( pConfig, x, y, z ); }; \
	}

	/*
	** Internal values for allowing extensions.
	*/
	AD_DEFAULT( "Internal", "URLProtocol", "http://" );
	AD_DEFAULT( "Internal", "DefaultPort", "80" );
	AD_DEFAULT( "Internal", "ServerIOObjectLine", "\tIOObject ServerIOObject" );
	AD_DEFAULT( "Internal", "DefaultMIMEType", "application/octet-stream" );
	AD_DEFAULT( "Internal", "HandlerLine", "\tHandlers Start Scripts WinScripts \
FastCGIScripts ISAPI Default" );

	/*
	** General defaults
	*/
	AD_DEFAULT( "General", "Threads", "20" );
	AD_DEFAULT( "General", "Admin", "webmaster@yourhost.dom" );
	AD_DEFAULT( "General", "ExePath", "C:\\Pi3Web\\bin\\EnhPi3.exe" );
	AD_DEFAULT( "General", "BinDir", "C:\\Pi3Web\\bin" );
	AD_DEFAULT( "General", "InstallDir", "C:\\Pi3Web" );
	AD_DEFAULT( "General", "SendTimeout", "350" );
	AD_DEFAULT( "General", "RecvTimeout", "400" );
	AD_DEFAULT( "General", "DNSReverseLookup", "" );
	AD_DEFAULT( "General", "RunMode", "Desktop" );
	AD_DEFAULT( "General", "ServiceName", "Pi3Web" );
	AD_DEFAULT( "General", "KeepOpen", "On" );

	/*
	** Identity defaults
	*/
	AD_DEFAULT( "General", "Port", "80" );
	AD_DEFAULT( "General", "Host", "localhost" );
	AD_DEFAULT( "Identity", "Use", "local" );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
void Internal_writeDefaultParagraphs( ostream &os )
{
#define SECTION_P16 "\n\
# ---\n\
# \n\
# Scripts, WinScripts, FastCGIScripts\n\
#\n\
#	Handler sequences for resources which have mapped to these\n\
# 	respective types\n\
#\n\
# ---\n\
<Object>\n\
	Name Scripts\n\
	Class FlexibleHandlerClass\n\
	Condition \"&cmp(&dblookup(response,string,ObjectMap),Scripts)\"\n\
	CheckPath RefuseFileByMask AllowFileMask=\"EFD\" RefuseStatus=404\n\
	CheckPath RefuseFileByMask AllowFileMask=\"F\" RefuseStatus=403\n\
	CheckPath ReturnCode ReturnCode=COMPLETED\n\
	CheckAccess AccessByFile RequirePermissions=\"X\"\n\
	CheckType ReturnCode ReturnCode=COMPLETED\n\
	Handle StandardCGI\n\
</Object>\n\
<Object>\n\
	Name WinScripts\n\
	Class FlexibleHandlerClass\n\
	Condition \"&cmp(&dblookup(response,string,ObjectMap),WinScripts)\"\n\
	CheckPath RefuseFileByMask AllowFileMask=\"EFD\" RefuseStatus=404\n\
	CheckPath RefuseFileByMask AllowFileMask=\"F\" RefuseStatus=403\n\
	CheckPath ReturnCode ReturnCode=COMPLETED\n\
	CheckAccess AccessByFile RequirePermissions=\"X\"\n\
	CheckType ReturnCode ReturnCode=COMPLETED\n\
	Handle WindowsCGI\n\
</Object>\n\
"
	os << SECTION_P16;

#define SECTION_P17 "\n\
<Object>\n\
	Name FastCGIScripts\n\
	Class FlexibleHandlerClass\n\
	Condition \"&cmp(&dblookup(response,string,ObjectMap),FastCGIScripts)\"\n\
	CheckPath RefuseFileByMask AllowFileMask=\"EFD\" RefuseStatus=404\n\
	CheckPath RefuseFileByMask AllowFileMask=\"F\" RefuseStatus=403\n\
	CheckPath ReturnCode ReturnCode=COMPLETED\n\
	CheckAccess ReturnCode ReturnCode=\"COMPLETED\"\n\
	CheckType ReturnCode ReturnCode=\"COMPLETED\"\n\
	Handle FastCGI \\\n\
		IOObject=\"FastCGIIOObject\" \\\n\
		Variable=\"AUTH_TYPE=$x\" \\\n\
		Variable=\"CONTENT_LENGTH=$Y\" \\\n\
		Variable=\"CONTENT_TYPE=$C\" \\\n\
		Variable=\"HTTPS=$g\" \\\n\
		Variable=\"&if($K,HTTPS_KEYSIZE=$K,)\" \\\n\
		Variable=\"&if($K,HTTPS_SECRETKEYSIZE=$G,)\" \\\n\
		Variable=\"PATH_INFO=$I\" \\\n\
		Variable=\"PATH_TRANSLATED=$Z\" \\\n\
		Variable=\"QUERY_STRING=$q\" \\\n\
		Variable=\"REMOTE_ADDR=$A\" \\\n\
		Variable=\"REMOTE_HOST=$h\" \\\n\
		Variable=\"REMOTE_IDENT=$i\" \\\n\
		Variable=\"REMOTE_USER=$u\" \\\n\
		Variable=\"REQUEST_METHOD=$m\" \\\n\
		Variable=\"SCRIPT_NAME=$z\" \\\n\
		Variable=\"SERVER_NAME=$v\" \\\n\
		Variable=\"SERVER_PORT=&if($p,$p,&if($K,443,80))\" \\\n\
		Variable=\"SERVER_PROTOCOL=$H\" \\\n\
		Variable=\"SERVER_SOFTWARE=$S\"\n\
</Object>\n\
<Object>\n\
	Name ISAPI\n\
	Class FlexibleHandlerClass\n\
	Condition \"&cmp(&dblookup(response,string,ObjectMap),ISAPI)\"\n\
	CheckPath RefuseFileByMask AllowFileMask=\"EFD\" RefuseStatus=404\n\
	CheckPath RefuseFileByMask AllowFileMask=\"F\" RefuseStatus=403\n\
	CheckPath ReturnCode ReturnCode=COMPLETED\n\
	CheckAccess AccessByFile RequirePermissions=\"X\"\n\
	CheckType ReturnCode ReturnCode=COMPLETED\n\
	Handle ISAPI20\n\
</Object>\n\
"
	os << SECTION_P17;

#define SECTION_P40 "\n\
# ---\n\
#\n\
# Directory listing with HTML tables\n\
#\n\
# ---\n\
<Object>\n\
	Name TableHTML\n\
	Class DirectoryIndexClass\n\
	HeaderPattern \"<HTML><TITLE>Index of %p</TITLE>\\\n\
<BODY BACKGROUND=\"/icons/\\\n\
&if(&cmpi($o,MainVirtualHostInformation),Pi3Tile.gif,H2Tile.gif)\"/*background depends on vhost*/\\\n\
BGCOLOR=\"#FFFFFF\">\\\n\
<H2>Index of %p</H2>\\n\"\n\
	ListTop \"<CENTER><TABLE CELLSPACING=0 CELLPADDING=0 WIDTH=100'%'>\\\n\
<TR>\\\n\
<TH NOWARP ALIGN=LEFT>\\\n\
<IMG SRC=\"/icons/blank.gif\" ALIGN=MIDDLE BORDER=NONE ALT=\"\">\\\n\
<TH NOWRAP ALIGN=LEFT>Name<BR><HR SIZE=1 NOSHADE>\\\n\
<TH NOWRAP ALIGN=LEFT>Size<BR><HR SIZE=1 NOSHADE>\\\n\
<TH NOWRAP ALIGN=LEFT>Last Modified<BR><HR SIZE=1 NOSHADE>\\\n\
<TH NOWRAP ALIGN=LEFT>Description<BR><HR SIZE=1 NOSHADE>\\n\"\n\
	FilePattern \"<TR>\\\n\
<TD NOWRAP ALIGN=LEFT VALIGN=TOP>\\\n\
<A HREF=\"%r\"><IMG SRC=\"%i\" BORDER=NONE ALIGN=MIDDLE ALT=\"\"></A>\\\n\
<TD NOWRAP ALIGN=LEFT VALIGN=TOP><A HREF=\"%r\">&abbrev(%n,20,...)</A>\\\n\
<TD NOWRAP ALIGN=LEFT VALIGN=TOP>%s<TD>%l\\\n\
<TD NOWRAP ALIGN=LEFT VALIGN=TOP>&if(%d,%d,<I>%c</I>)\\n\"\n\
	ListBottom \"</TABLE></CENTER>\\n\"\n\
	LastModifiedFormat \"%d-%b-%y %H:%M\"\n\
	Include \"*\"\n\
\n"
#define SECTION_P41 "\
	MIMEIcon \"text/html /icons/layout.gif\"\n\
	MIMEIcon \"text/x-code /icons/c.gif\"\n\
	MIMEIcon \"text/x-perl /icons/p.gif\"\n\
	MIMEIcon \"text/\* /icons/text.gif\"\n\
	MIMEIcon \"internal/x-directory /icons/folder.gif\"\n\
	MIMEIcon \"audio/\* /icons/sound2.gif\"\n\
	MIMEIcon \"image/\* /icons/image2.gif\"\n\
	MIMEIcon \"application/x-uuencoded /icons/uuencoded.gif\"\n\
	MIMEIcon \"application/x-tar /icons/compressed.gif\"\n\
	MIMEIcon \"application/x-gzip /icons/compressed.gif\"\n\
	MIMEIcon \"application/x-sh /icons/script.gif\"\n\
	MIMEIcon \"application/\* /icons/binary.gif\"\n\
	MIMEIcon \"x-world/x-vrml /icons/burst.gif\"\n\
	MIMEIcon \"\*/\* /icons/unknown.gif\"\n\
	DescriptionFile \".desc\"\n\
	SwapFileName \".. Parent Directory\"\n\
	FooterPattern \"<HR SIZE=1 NOSHADE>\\\n\
Contact the hostmaster <A HREF=\"mailto:$a\"<I>$a</I></A>.\\\n\
</BODY></HTML>\"\n\
	HeaderFile \"HEADER\"\n\
	FooterFile \"README\"\n\
	Exclude \".\"\n\
	Exclude \"~*\"\n\
	Exclude \".desc\"\n\
	Exclude \"README\"\n\
	Exclude \"HEADER\"\n\
	Options \" Name | AbbrevSize | RelPath | Icon | MIMEType \"\n\
</Object>\
\n"
	os << SECTION_P40;
	os << SECTION_P41;

#define SECTION_P45 "\n\
# ---\n\
#\n\
# CGIFileIOObject\n\
# 	\n\
#	IOObject used for communication between web browser and\n\
#	server.\n\
#\n\
# ---\n\
<Object>\n\
	Name CGIFileIOObject\n\
	Class PipeIOClass\n\
	ReadTimeout -1\n\
	WriteTimeout 10\n\
	NoYield \"Read\"\n\
</Object>\n\
\n"
	os << SECTION_P45;

#define SECTION_P20 "\n\
# ---\n\
#\n\
# StandardCGI\n\
#\n\
# Configuration for Standard CGI\n\
#\n\
# ---\n\
<Object>\n\
	Name StandardCGI\n\
	Class CGIClass\n\
	\n\
	# Specify the IO object use to communicate with the CGI program\n\
	FileIOObject CGIFileIOObject\n\
\n\
	# Snoop IO objects can be instantiated in the IO chain with a line\n\
	# like this. They log all IO trafic. This is useful for debugging.\n\
#	FileIOObject SnoopIO IOObject=\"CGIFileIOObject\" OutFile=\"cgi.dbg\" \n\
\n\
	DefaultCommandLine \"%p%q\"\n\
	CommandLineByExt .bat=\"cmd.exe /c %p%q\"\n\
	CommandLineByExt .cmd=\"cmd.exe /c %p%q\"\n\
	CommandLineByExt .pl=\"perl %p%q\"\n\
	IncludeParentsEnvironment Yes\n\
	EnvironmentSize 8192\n\
	\n\
	# These are the environment variables that are sent to the\n\
	# CGI program\n\
	Variable \"AUTH_TYPE=$x\"\n\
	Variable \"CONTENT_LENGTH=$Y\"\n\
	Variable \"CONTENT_TYPE=$C\"\n\
	Variable \"HTTPS=$g\"\n\
	Variable \"&if($K,HTTPS_KEYSIZE=$K,)\"\n\
	Variable \"&if($K,HTTPS_SECRETKEYSIZE=$G,)\"\n\
	Variable \"GATEWAY_INTERFACE=CGI/1.1\"\n\
	Variable \"PATH_INFO=$I\"\n\
	Variable \"PATH_TRANSLATED=$Z\"\n\
	Variable \"QUERY_STRING=$q\"\n\
	Variable \"REMOTE_ADDR=$A\"\n\
	Variable \"REMOTE_HOST=$h\"\n\
	Variable \"REMOTE_IDENT=$i\"\n\
	Variable \"REMOTE_USER=$u\"\n\
	Variable \"REQUEST_METHOD=$m\"\n\
	Variable \"SCRIPT_NAME=$z\"\n\
	Variable \"SERVER_NAME=$v\"\n\
	Variable \"SERVER_PORT=&if($p,$p,&if($K,443,80))\"\n\
	Variable \"SERVER_PROTOCOL=$H\"\n\
	Variable \"SERVER_SOFTWARE=$S\"\n\
	ExtraHeaders Yes\n\
	ExtraHeadersPrefix \"HTTP_\"\n\
	ExtraHeadersIgnore \"Content-Type Content-Length\"\n\
#	SendCRLF Yes\n\
    #\n\
    # Kill CGI programs after 1 minute if they don't respond\n\
    #\n\
    KillAfter 60\n\
</Object>\n\
\n"
	os << SECTION_P20;

#define SECTION_P21 "\n\
# ---\n\
#\n\
# WindowsCGI\n\
#\n\
# Handle windows CGI\n\
#\n\
# ---\n\
<Object>\n\
	Name WindowsCGI\n\
	Class WinCGIClass\n\
	FileIOObject CGIFileIOObject\n\
	DefaultCommandLine \"%p %d\"\n\
	IncludeParentsEnvironment Yes\n\
	EnvironmentSize 8192\n\
	Variable \"AUTH_TYPE=$x\"\n\
	Variable \"CONTENT_LENGTH=$Y\"\n\
	Variable \"CONTENT_TYPE=$C\"\n\
	Variable \"HTTPS=$g\"\n\
	Variable \"&if($K,HTTPS_KEYSIZE=$K,)\"\n\
	Variable \"&if($K,HTTPS_SECRETKEYSIZE=$G,)\"\n\
	Variable \"GATEWAY_INTERFACE=CGI/1.1\"\n\
	Variable \"PATH_INFO=$I\"\n\
	Variable \"PATH_TRANSLATED=$Z\"\n\
	Variable \"QUERY_STRING=$q\"\n\
	Variable \"REMOTE_ADDR=$A\"\n\
	Variable \"REMOTE_HOST=$h\"\n\
	Variable \"REMOTE_IDENT=$i\"\n\
	Variable \"REMOTE_USER=$u\"\n\
	Variable \"REQUEST_METHOD=$m\"\n\
	Variable \"SCRIPT_NAME=$z\"\n\
	Variable \"SERVER_NAME=$v\"\n\
	Variable \"SERVER_PORT=&if($p,$p,&if($K,443,80))\"\n\
	Variable \"SERVER_PROTOCOL=$H\"\n\
	Variable \"SERVER_SOFTWARE=$S\"\n\
	ExtraHeaders Yes\n\
	ExtraHeadersPrefix \"HTTP_\"\n\
	ExtraHeadersIgnore \"Content-Type Content-Length\"\n\
\n\
	# Generate windows CGI filenames by preappending the\n\
	# process id to the unique identifier.\n\
	DataFile \"cgi-temp/$P_%u.ini\"\n\
	StdinFile \"cgi-temp/$P_%u.in\"\n\
	StdoutFile \"cgi-temp/$P_%u.out\"\n\
	DataBlock \"\\\
[CGI]$M\\\n\
Request Protocol=$H$M\\\n\
Request Method=$m$M\\\n\
Physical Path=$f$M\\\n\
CGI Version=CGI/1.1 (Win)$M\\\n\
Content Type=$C/* short for  &dblookup(request,rfc822,Content-Type)*/$M\\\n\
Content Length=$Y/* short for &dblookup(request,rfc822,Content-Length)*/$M\\\n\
$M\\\n\
[Accept]$M\\\n\
$M\\\n\
[System]$M\\\n\
Output File=%o$M\\\n\
Content File=%i$M\\\n\
$M\\\n\
[Accept]$M\\\n\
$M\\\n\
[Extra Headers]$M\\\n\
$M\\\n\
[Form Literal]$M\\\n\
$M\\\n\
[Form External]$M\\\n\
$M\\\n\
[Form File]$M\\\n\
$M\\\n\
[Form Huge]$M\\n\\
$M\\\n\
\"\n\
#	SendCRLF Yes\n\
</Object>\n\
\n"
	os << SECTION_P21;
	
#define SECTION_P50 "\n\
# ---\n\
#\n\
# Documentation is handled as HTML regardless of file extension. Also,\n\
# files have headers and footers attached to make them pretty.\n\
#\n\
# ---\n\
<Object>\n\
	Name DocumentMapper\n\
	Class PathMapperClass\n\
	#\n\
	# leave calling card\n\
	#\n\
	Action \"&dbreplace(response,string,ObjectMap,Pi3Docs)\"\n\
</Object>\n\
\n\
# ---\n\
#\n\
# CGIMapper\n\
#\n\
#	Instance of PathMapperClass to Map CGI URLs\n\
#\n\
#	CGI Scripts set PATH_INFO. \n\
#\n\
# ---\n\
<Object>\n\
	Name CGIMapper\n\
	Class PathMapperClass\n\
	CaseSensitive \"No\"\n\
	#\n\
	# Subdirectories below the mapping directory should be put in path info\n\
	#\n\
	PathInfo \"Yes\"\n\
	#\n\
	# Set the 'ObjectMap' variable in the response DB if this map is\n\
	# successful\n\
	#\n\
	Action \"&dbreplace(response,string,ObjectMap,Scripts)\"\n\
</Object>\n\
\n\
# ---\n\
#\n\
# WinCGIMapper, FCGIMapper and ISAPIMapper\n\
#\n\
# 	Like CGIMapper, except it leaves a different marker to cause\n\
# 	a different object to handle the request.\n\
#\n\
# ---\n\
<Object>\n\
	Name WinCGIMapper\n\
	Class PathMapperClass\n\
	PathInfo \"Yes\"\n\
	Action \"&dbreplace(response,string,ObjectMap,WinScripts)\"\n\
</Object>\n\
<Object>\n\
	Name FCGIMapper\n\
	Class PathMapperClass\n\
	PathInfo \"Yes\"\n\
	Action \"&dbreplace(response,string,ObjectMap,FastCGIScripts)\"\n\
</Object>\n\
<Object>\n\
	Name ISAPIMapper\n\
	Class PathMapperClass\n\
	PathInfo \"Yes\"\n\
	Action \"&dbreplace(response,string,ObjectMap,ISAPI)\"\n\
</Object>\n\
\n"
	os << SECTION_P50;

#define SECTION_P51 "\n\
# ---\n\
#\n\
# FastCGIIOObject\n\
# \n\
# IOObject for communicating with fast CGI application servers. This\n\
# one uses the TCP/IP transport.\n\
#\n\
# ---\n\
<Object>\n\
	Name FastCGIIOObject\n\
	Class TCPIPIOClass\n\
	Type Active\n\
\n\
	# Timeouts. -1 specifies no timeout.\n\
	RecvTimeout 20\n\
	SendTimeout 20\n\
</Object>\n\
\n"
	os << SECTION_P51;

#define SECTION_P52 "\n\
# ---\n\
#\n\
# AccessLogger\n\
#\n\
#	An instance of ExpressionLoggerClass configured to generate an \n\
#	logfile\n\
#\n\
# ---\n\
<Object>\n\
	Name AccessLogger\n\
	Class ExpressionLoggerClass\n\
\n\
	#\n\
	# Append\n\
	#\n\
	OpenMode \"a\"\n\
	#\n\
	# Line to log \n\
	Expression \"$A $h - [$t] \\\"$r\\\" $s $b$M\"\n\
</Object>\n\
\n\
# ---\n\
#\n\
# RefererLogger\n\
#\n\
#	Log bad links only (HTTP error code 404)\n\
#\n\
# ---\n\
<Object>\n\
	Name RefererLogger\n\
	Class ExpressionLoggerClass\n\
\n\
	#\n\
	# Append\n\
	#\n\
	OpenMode \"a\"\n\
	#\n\
	# Line to log \n\
	Expression \"&if(&cmp($s,404),\\\n\
[$t] $h \\\"$r\\\" &dblookup(request,rfc822,Referer)$M)\"\n\
</Object>\n\
\n"
	os << SECTION_P52;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
void Internal_writeCustomizedParagraphs( QConfig *pConfig, ostream &os )
{
    int iTmp;

#define L(x,y) (M_LookupValue(pConfig, x, y))

#define SECTION_P1 "\
# ===\n\
#\n\
#	AUTOMATICALLY GENERATED FILE\n\
#\n\
#	This files was automatically generated by the Pi3 enhanced GUI\n\
#	(EnhPi3.exe) for Windows 95 and Window NT 4.0\n\
#\n\
#	Any changes you make to this file will be lost next time the \n\
#	configuration GUI (server admin) is run and changes \n\
#	are saved.\n\
#\n\
# ===\n\
\n\
\n\
\n\
# ===\n\
#\n\
# Top level configuration\n\
#\n\
#	** IMPORTANT **\n\
#	If you do nothing else then do this:\n\
#\n\
\n\
#... ### ### Customize server settings\n\
\n\
# Skip below to the definition of 'MainVirtualHostInformation', change\n\
# the settings for Hostname (*Mandatory* redirects will not work without\n\
# this!) and Administrator (optional).\n\
#\n\
# ===\n\
\n\
# Include  standard IO and server stuff\n\
include ../Conf/IO.cnf\n\
include ../Conf/Server.cnf\n\
include ../Conf/Pi3API.cnf\n\
include ../Conf/HTTP.cnf\n\
include ../Conf/Fcgi.cnf\n\
include ../Conf/Plugins.cnf\n\
include ../Conf/ISAPI.cnf\n\
\n\
# ---\n\
#\n\
# IO Object for main listening socket.\n\
#\n\
# ---\n\
<Object>\n\
    Name ServerIOObject\n\
    Class TCPIPIOClass\n\
    Type Passive\n\
\n\
# A hostname must be specified for the Pi3Web HTTP server. The hostname\n\
# will be the used by the server to identify itself in URLs generated\n\
# for redirects and will also be used to lookup the IP address that this\n\
# server will listen on.\n\
\n"
	/*
	** Write network preferences
	*/
	os << SECTION_P1;
	os << "\
	BindHost " << L("General", "Host") << "\n\
\n\
# A TCP port must be specified for this HTTP server. The TCP port\n\
# is the port that the HTTP will listen for requests on. This is\n\
# typically 80 for HTTP servers. \n\
#\n\
# Note that UNIX operating systems normally require that the server\n\
# initially run as user 'root' to bind on ports between 0 and 1024\n\
	BindPort " << L("General", "Port") << "\n\
\n\
# Timeout accepting connections. -1 specifies no timeout.\n\
	AcceptTimeout -1\n\
\n\
# Specify timeout for sending data. -1 specifies no timeout. \n\
	RecvTimeout " << L("General", "RecvTimeout" ) << "\n\
\n\
# Specify timeout for receiving data. -1 specifies no timeout. \n\
	SendTimeout " << L("General", "SendTimeout" ) << "\n" << endl;

#define SECTION_P1_2 "\
	PeerIPVariable STRING:RemoteAddr\n\
	PeerHostNameVariable STRING:RemoteHost\n\
	LocalAddressVariable STRING:LocalAddr\n\
\n\
# Flags\n\
#\n\
#   OwnDB - Object adds its own DB, rather than use the parents\n\
#               this is necessary to avoid collision of values placed in\n\
#               the DB between different connection. \n\
#\n\
#   SetPeerAddress - IP address of remote peer with be put\n\
#               into the specified variable.\n\
#\n\
#   DNSReverseLookup - Use reverse DNS to get the hostname of the \n\
#               peer and place it in the specified variable.\n\
#\n\
#   SetLocalAddress - Put local address in the specified variable.\n\
"

	os << SECTION_P1_2;
	os << "\tFlags OwnDB|";
	if ( *M_LookupValue( pConfig, "General", "DNSReverseLookup" ) )
		{ os << "DNSReverseLookup|"; };
	os << "SetPeerAddress|SetLocalAddress" << endl;
	os << "</Object>" << endl << endl;

/*
** Write out information on the main virtual host
*/
#define SECTION_P2 "\
# ---\n\
# Main virtual host object.\n\
# ---\n\
<Object>\n\
	Name MainVirtualHostInformation\n\
	Class VirtualHostClass\n"

	os << SECTION_P2;
	/*
	** Main virtual host settings
	*/
	os << "\tAdministrator " << L( "General", "Admin" ) << endl;
	os << "\tHostName " << L( "General", "Host" ) << endl;
	const char *pPort = L( "General", "Port" );
	if ( atoi( pPort )!=80 )
		{ os << "\tServerPort " << L( "General", "Port" ) << endl; };
    os << "</Object>" << endl;

    /*
    ** Now do ip virtual host objects
    */
	iTmp = 0;
	for(;; iTmp++ )
		{
		const char *pTmp = M_LookupValueEx( pConfig, "VirtualHosts", 
					"IP", iTmp );
		if ( !pTmp ) { break; };

		/*
		** Split the virtual host into IP, hostname, docroot, administrator
		*/
		int i;
		for( i=0; pTmp[i] && pTmp[i]!='\t'; i++);
		PIString sIPAddress( pTmp, i );

		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && isspace(pTmp[i]); i++);
		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && pTmp[i]!='\t'; i++);
		PIString sHostname( pTmp, i );

		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && isspace(pTmp[i]); i++);
		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && pTmp[i]!='\t'; i++);
		PIString sDocroot( pTmp, i );

		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && isspace(pTmp[i]); i++);
		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && pTmp[i]!='\t'; i++);
		PIString sAdministrator( pTmp, i );

        /*
        ** Write out the details
        */
        os << "# --- The IP virtual host '" << sIPAddress << "' ---" << endl;
        os << "<Object>" << endl;
        os << "\tName " << sIPAddress << endl;
        os << "\tClass VirtualHostClass" << endl;
        os << "\tAdministrator " << sAdministrator << endl;
        os << "\tHostName " << sHostname << endl;
		if ( atoi( pPort )!=80 )
			{ os << "\tServerPort " << L( "General", "Port" ) << endl; };
        os << "</Object>" << endl;
        };

    /*
    ** Now do non-ip virtual host objects
    */
	iTmp = 0;
	for(;; iTmp++ )
		{
		const char *pTmp = M_LookupValueEx( pConfig, "VirtualHosts", 
					"NonIP", iTmp );
		if ( !pTmp ) { break; };

		/*
		** Split the virtual host into hostname, docroot, administrator
		*/
		int i;
		for( i=0; pTmp[i] && pTmp[i]!='\t'; i++);
		PIString sHostname( pTmp, i );

		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && isspace(pTmp[i]); i++);
		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && pTmp[i]!='\t'; i++);
		PIString sDocroot( pTmp, i );

		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && isspace(pTmp[i]); i++);
		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && pTmp[i]!='\t'; i++);
		PIString sAdministrator( pTmp, i );

        /*
        ** Write out the details
        */
        os << "# --- The Non-IP virtual host '" << sHostname << "' ---" << endl;
        os << "<Object>" << endl;
        os << "\tName " << sHostname << endl;
        os << "\tClass VirtualHostClass" << endl;
        os << "\tAdministrator " << sAdministrator << endl;
        os << "\tHostName " << sHostname << endl;
		if ( atoi( pPort )!=80 )
			{ os << "\tServerPort " << L( "General", "Port" ) << endl; };
        os << "</Object>" << endl;
        };

#define SECTION_P3 "\
\n\
# ---\n\
#\n\
#    Thread dispatch object\n\
#\n\
#   This is an instance of MultiThreadedIOServerClass with appropriate\n\
#   parameters\n\
#\n\
# ---\n\
<Object>\n\
	Name ThreadPoolDispatcher\n\
	Class MultiThreadedIOServerClass\n\
\n\
# The IOObject is the object that will be used for the listening socket\n\
# it can be a real IOObject (such as TCP/IP), as test IO object which \n\
# feeds stub data into the server or a filter IO object which encrpts\n\
# or logs data passed through it.\n\
#\n\
#    Example test IO object:\n\
# 	IOObject TestIOObject\n\
#\n\
#    Log IO to file:\n\
#	IOObject SnoopIO IOObject=\"ServerIOObject\" OutFile=\"./snoop.dbg\"\n\
#\n\
"
#define SECTION_P3_1 "\
\n\
# Dispatch accepted IO connection to the HTTPLogicObject\n\
	LogicObject HTTPLogicObject\n\
\n\
# Number of threads to use \n\
\n"
#define SECTION_P3_2 "\n\
\n\
# Number of requests to exit after\n\
	ExitAfter 1000\n\
\n\
</Object>\n\
\n"

#define SECTION_P4 "\n\
# ---\n\
#\n\
# Main Pi3 object. \n\
#\n\
# This is an instance of the UNIXDaemonClass to fork the server into\n\
# one monitor process and 'NumberOfProcesses' child processes. \n\
#\n\
# The monitor process automatically starts a new child processes whenever\n\
# a child process exits.\n\
#\n\
# ---\n\
<Object>\n\
	Name Main\n\
	Class UNIXDaemonClass\n\
\n\
# Actual class to accept new IO requests and dispatch them\n\
	ServerObject ThreadPoolDispatcher\n\
	PIDFile \"../Logs/pid.log\"\n\
	RLimitCpu Maximum\n\
\n\
# User that child processes will run as. Server must run as root to \n\
# be able to change to the specified user. A user id number can be\n\
# specified with #number. Server which start with root privileges should\n\
# change to a user id with lesser privileges for security reasons.\n\
#   User nobody\n\
\n\
# Group that child processes will run as. Server must run as root to\n\
# be able to change to a different group. A group id number can be\n\
# specified with #number.\n\
#   Group #-1\n\
\n\
# generate a server unique lockfile name and write it into\n\
# Logs/lockfile.txt\n\
	LockFileName \"../Logs/lockfile.txt\"\n\
\n\
# Message to write just before the server is ready to start accepting\n\
# connections.\n\
# default\n\
	Message \"Pi3Web HTTP Server Started\"\n\
\n\
# Specify the number of processes. \n\
\n\
	NumberOfProcesses 8\n\
</Object>\n\
\n"

	os << SECTION_P3;
	os << L("Internal", "ServerIOObjectLine" ) << endl;
	os << SECTION_P3_1;
	os << "\tMaxThreads " << L( "General", "Threads" );
	os << SECTION_P3_2;
	os << SECTION_P4;

#define SECTION_P10 "\n\
# ---\n\
#\n\
# Start\n\
#\n\
#	Handle initial phases of requests, typically before we have enough\n\
#	information to select different execution paths\n\
#\n\
# ---\n\
<Object>\n\
	Name Start\n\
	Class FlexibleHandlerClass\n\
	Init ReturnCode ReturnCode=COMPLETED\n\
	Headers ReadRequestHeaders\n\
"
    os << SECTION_P10;

    /*
    ** Write out the HOSTMAP phases for IP virtual hosts
    */
	iTmp = 0;
	for(;; iTmp++ )
		{
		const char *pTmp = M_LookupValueEx( pConfig, "VirtualHosts", 
					"IP", iTmp );
		if ( !pTmp ) { break; };

		/*
		** Split the virtual host into IP, hostname, docroot, administrator
		*/
		int i;
		for( i=0; pTmp[i] && pTmp[i]!='\t'; i++);
		PIString sIPAddress( pTmp, i );

        os << "\tHostMap VirtualHostByAddress VirtualHostObject=\""
            << sIPAddress << "\" BindAddress=\"" << sIPAddress << "\"" << endl;
        };

    /*
    ** Write out the HOSTMAP phases for Non-IP virtual hosts
    */
	iTmp = 0;
	for(;; iTmp++ )
		{
		const char *pTmp = M_LookupValueEx( pConfig, "VirtualHosts", 
					"NonIP", iTmp );
		if ( !pTmp ) { break; };

		/*
		** Split the virtual host into [IP], hostname, docroot, [administrator]
		*/
		int i;
		for( i=0; pTmp[i] && pTmp[i]!='\t'; i++);
		PIString sHostname( pTmp, i );

        os << "\tHostMap VirtualHostByName VirtualHostObject=\"" << sHostname
            << "\" HostName=\"" << sHostname << "\"" << endl;
        };

#define SECTION_P10_1 "\n\
	HostMap ReturnCode ReturnCode=COMPLETED\n\
#	Mapping DirectoryMappings\n\
\n\
	#\n\
	# Default mappings\n\
	#\n\
	Mapping MapToErrorDocument \\\n\
		IgnoreStatus=\"0 200\" \\\n\
		DefaultMessage=\"/errors/unknown.htm\" \\\n\
		301=\"/errors/301.ssi\" \\\n\
		304=\"/errors/304.ssi\" \\\n\
		400=\"/errors/400.htm\" \\\n\
		401=\"/errors/401.ssi\" \\\n\
		403=\"/errors/403.ssi\" \\\n\
		404=\"/errors/404.ssi\" \\\n\
		500=\"/errors/500.ssi\" \\\n\
		501=\"/errors/501.htm\"\n\
#	Mapping UserDirectory UserDirectoryRoot=\"/home/\" \\\n\
#		HTMLDirectory=\"/public_html\"\n\
"
	os << SECTION_P10_1;

    /*
    ** Place holders for default document root
    */
    int iIsDocumentRoot;
    PIString sDocumentRootVirtualPath;
    const char *pDocumentRootType;
    PIString sDocumentRootRealm;

	/*
	** Write in mappings
	*/
	iTmp = 0;
	for(;; iTmp++ )
		{
		const char *pTmp = M_LookupValueEx( pConfig, "Mappings", 
					"Mapping", iTmp );
		if ( !pTmp ) { break; };

        iIsDocumentRoot = 0;
        sDocumentRootVirtualPath = PIString::Empty();
        pDocumentRootType = 0;
        sDocumentRootRealm = PIString::Empty();

		/*
		** Split the mappings into type, from and to
		*/
		int i;
		for( i=0; pTmp[i] && pTmp[i]!='\t'; i++);
		PIString sType( pTmp, i );

		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && isspace(pTmp[i]); i++);
		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && pTmp[i]!='\t'; i++);
		PIString sFrom( pTmp, i );

        /*
        ** See if this is the document root
        */
        if ( sFrom=="/" )
            {
            iIsDocumentRoot = 1;
            };

		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && isspace(pTmp[i]); i++);
		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && pTmp[i]!='\t'; i++);
		PIString sTo( pTmp, i );

		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && isspace(pTmp[i]); i++);
		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && pTmp[i]!='\t'; i++);
		PIString sRealm( pTmp, i );

		if ( sType=="CGI" )
			{ pDocumentRootType = "CGIMapper"; }
		else if ( sType=="Documents" )
			{ pDocumentRootType = "PathMapper"; }
		else if ( sType=="Windows CGI" )
			{ pDocumentRootType = "WinCGIMapper"; }
		else if ( sType=="Fast CGI" )
			{ pDocumentRootType = "FCGIMapper"; }
		else if ( sType=="ISAPI" )
			{ pDocumentRootType = "ISAPIMapper"; }
		else
			{ assert( 0 ); };

        if ( !iIsDocumentRoot )
            {
            os << "\tMapping " << pDocumentRootType << " ";
            pDocumentRootType = 0;
            };

		/*
		** Write in code to generate a realm marker	
		*/
        if ( iIsDocumentRoot )
            {
            sDocumentRootVirtualPath = sTo;
            }
        else
            {
		    os << "From=\"" << sFrom << "\" To=\"" << sTo << "\" ";
            };
		if ( strcmp( sRealm, "(none)" ) )
			{
            if ( !iIsDocumentRoot )
                {
                os << "\\" << endl;
                os << "\t\tAction=\"&dbreplace(response,string,AuthenticationRealm,";
                os << sRealm << ")\"";
                }
            else
                { sDocumentRootRealm = sRealm; };
			};
		os << endl;
		};
   
    /*
    ** Write IP hosts
    */
	iTmp = 0;
	for(;; iTmp++ )
		{
		const char *pTmp = M_LookupValueEx( pConfig, "VirtualHosts", 
					"IP", iTmp );
		if ( !pTmp ) { break; };

		/*
		** Split the virtual host into IP, hostname, docroot, administrator
		*/
		int i;
		for( i=0; pTmp[i] && pTmp[i]!='\t'; i++);
		PIString sIPAddress( pTmp, i );

		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && isspace(pTmp[i]); i++);
		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && pTmp[i]!='\t'; i++);

		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && isspace(pTmp[i]); i++);
		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && pTmp[i]!='\t'; i++);

		PIString sDocroot( pTmp, i );

        os << "\t# Document root for the IP virtual host '"
            << sIPAddress << "'" << endl;
        os << "\tMapping Condition=\"&cmp($l," << sIPAddress << ")\" \\"
            << endl;
        os << "\t\tPathMapper From=\"/\" To=\"" << sDocroot << "\"" << endl;
        };

    /*
    ** Write NonIP hosts
    */
	iTmp = 0;
	for(;; iTmp++ )
		{
		const char *pTmp = M_LookupValueEx( pConfig, "VirtualHosts", 
					"NonIP", iTmp );
		if ( !pTmp ) { break; };

		/*
		** Split the virtual host into hostname, docroot, administrator
		*/
		int i;
		for( i=0; pTmp[i] && pTmp[i]!='\t'; i++);
		PIString sHostname( pTmp, i );

		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && isspace(pTmp[i]); i++);
		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && pTmp[i]!='\t'; i++);
		PIString sDocroot( pTmp, i );

        os << "\t# Document root for the Non-IP virtual host '"
            << sHostname << "'" << endl;
        os << "\tMapping Condition=\"&cmpi($v," << sHostname << ")\" \\" <<endl;
        os << "\t\tPathMapper From=\"/\" To=\"" << sDocroot << "\"" << endl;
        };

    /*
    ** Write document root 
    */
    os << "#" << endl;
    os << "# Default document root" << endl;
    os << "#" << endl;
    if ( iIsDocumentRoot )
        {
        os << "\tMapping " << pDocumentRootType << " ";
        os << "From=\"/\" To=\"" << sDocumentRootVirtualPath << "\" ";
        if ( sDocumentRootRealm!=PIString::Empty() )
            {
            os << "\\" << endl;
            os <<"\t\tAction=\"&dbreplace(response,string,AuthenticationRealm,";
            os << sDocumentRootRealm << ")\"";
            };

		os << endl;
        };

#define SECTION_P11 "\
</Object>\n\
\n"
	os << SECTION_P11;

#define SECTION_P15 "\n\
# ---\n\
#\n\
# Default\n\
#\n\
#	Handle requests that other handlers have passed up\n\
#\n\
# ---\n\
<Object>\n\
	Name Default\n\
	Class FlexibleHandlerClass\n\
	CheckPath DirectoryExists IndexFile=\"index.ssi\" IndexFile=\"index.html\" IndexFile=\"index.htm\" IndexFile=\"default.ssi\" IndexFile=\"default.htm\" IndexFile=\"default.html\"\n\
	CheckPath RefuseFileByMask AllowFileMask=\"EF\" RefuseStatus=404\n\
	CheckPath RefuseFileByMask AllowFileMask=\"F\" RefuseStatus=403\n\
	CheckPath ReturnCode ReturnCode=COMPLETED\n\
	CheckAuth Authenticate\n\
	CheckAccess AccessByFile RequirePermissions=\"R\"\n\
\n\
	#\n\
	# Cause a '501 Not Implemented' Message for requests that\n\
	# are not 'GET' or 'HEAD', i.e.\n\
	#	if !( Method=='GET' || Method=='HEAD' ) \n\
	#	then\n\
	#		set status = 501\n\
	#	endif\n\
	#\n\
	CheckType Condition=\"&not(&or(&cmpi($m,GET),&cmpi($m,HEAD)))\" \\\n\
		StatusCode StatusCode=\"501\"\n\
#	CheckType ConditionalGet Flag=\"RefreshOlderThanServer\"\n\
	CheckType ConditionalGet \n\
	CheckType CheckForDirectory SetType=\"internal/x-directory\"\n\
\n\
	#\n\
	# Otherwise: Set the media type based on the file extension\n\
	#\n\
	CheckType MIMETypeByExtension \n\
\n\
	#\n\
	# Return 'COMPLETED' so an internal error will not be raised\n\
	#\n\
	CheckType ReturnCode ReturnCode=\"COMPLETED\"\n\
\n\
	#\n\
	# Most requests refer to a file, so put that first \n\
	#\n\
	# Handle the request by sending a file if\n\
	#	- The Value of ObjectMap in the response DB is \"\"\n\
	#	AND\n\
	#	- The Media type of the resource does not match internal/* \n\
	#\n\
	Handle Condition=\"&and(&not(&dblookup(response,string,ObjectMap)),\\\n\
&not(&regexp('internal/*',$c)))\" SendFile\n\
	Handle Condition=\"&cmp($c,internal/x-directory)\" TableHTML\n\
	Handle Condition=\"&cmp($c,internal/x-server-parsed-html)\" SendSSI \\\n\
		Exec=Yes ExecFileFixup=\"\\n\
/* exec file=\"\" maps to Standard CGI (Scripts) */\\\n\
&dbreplace(response,string,ObjectMap,Scripts)\\n\
&dbreplace(response,rfc822,Content-Type,internal/x-cgi)\"\n\
	Handle Condition=\"&cmp($c,internal/x-imagemap)\" ImageMap\n\
	Handle SendFile\n\
"
	os << SECTION_P15;

	/*
	** General logfiles
	*/
	/*
	** Write in mappings
	*/
	iTmp = 0;
	for(;; iTmp++ )
		{
		const char *pTmp = M_LookupValueEx( pConfig, "Logfiles", 
					"Logging", iTmp );
		if ( !pTmp ) { break; };

		/*
		** Split the logfiles into type and path
		*/
		int i;
		for( i=0; pTmp[i] && pTmp[i]!='\t'; i++);
		PIString sType( pTmp, i );
		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && isspace(pTmp[i]); i++);
		pTmp = &(pTmp[i]);
		PIString sPath( pTmp );

		if ( sType=="Access (CLF)" )
			{ os << "	Log AccessLogger "; }
		else if ( sType=="Referer Logfile" )
			{ os << "	Log RefererLogger "; }

		os << "File=\"" << sPath << "\" \n";
		};
	os << "	Log ReturnCode ReturnCode=COMPLETED\n\
	Destroy DeleteTemporaryFiles\n\
</Object>\n\
\n";

#define SECTION_P29_1 "\n\
# ---\n\
#\n\
# ProcessSite object, root for this website\n\
#\n\
# ---\n\
<Object>\n\
    Name ProcessSite\n\
    Class FlexibleHandlerClass\n\
";
#define SECTION_P29_2 "\n\
</Object>\n\
\n";

	os << SECTION_P29_1;
	os << M_LookupValue( pConfig, "Internal", "HandlerLine" ) << endl;
	os << SECTION_P29_2;

#define SECTION_P30 "\n\
# ---\n\
#\n\
# HTTPLogicObject\n\
#\n\
# 	HTTP Logic object, dispatches requests to all other handlers\n\
#\n\
# ---\n\
<Object>\n\
	Name HTTPLogicObject\n\
	Class HTTPDispatcherClass\n\
"

#define SECTION_P30_05 "\n\
	ServerRoot ./../\n\
	ServerStamp Pi3Web/1.0.2\n\
	DefaultHost MainVirtualHostInformation\n\
# MIMEFile is not used here because extensions to media mappings are done \n\
# explicitly with AddMIMEEntry \n\
#	MIMEFile \"Fragment/Mime.typ\"\n\
	ErrorLogFile \"Logs/error.txt\"\n\
"

#define SECTION_P30_1 "\
\n\
	# Specifying a file for DebugLogFile effectively turns on debugging\n\
#	DebugLogFile \"Logs/debug.txt\"\n\
\n\
	#\n\
	# The following to directives specify expressions to be logged\n\
	# before and after each handler execution respectively.\n\
	# The expression as given logs process/thread and handler context\n\
	# information indented to the nested handler level.\n\
	#\n\
	# Timing information is also given for performance tuning.\n\
	#\n\
	# This logging information is EXTREMELY verbose. Don't even\n\
	# think about using it in a production server\n\
	#\n\
	DebugBeforeHandler \"[$P:$k] &align(,$X)&align(,$X) |--> $T \\\n\
-------- &align($N,12) &align($R,12) $n \\\"$r\\\" $s\"\n\
	DebugAfterHandler \"[$P:$k] &align(,$X)&align(,$X) |<-- $T \\\n\
&align($D,8) &align($N,12) &align($R,12) $n \\\"$r\\\" $s\"\n\
</Object>\n\
"
	os << SECTION_P30;

	/*
	** Write in MIME types
	*/
	iTmp = 0;
	for(;; iTmp++ )
		{
		const char *pTmp = M_LookupValueEx( pConfig, "MIMETypes", 
					"Mapping", iTmp );
		if ( !pTmp ) { break; };

		os << "\tAddMIMEEntry \"" << pTmp << "\"" << endl;
		};

	os << "\tKeepOpen \"" << M_LookupValue( pConfig, "General", "KeepOpen" ) 
			<< '"' << endl;
	os << "\tDefaultMIMEType \"" << M_LookupValue( pConfig, "Internal", 
		"DefaultMIMEType" ) << "\"" << endl;
	os << SECTION_P30_05;
	os << M_LookupValue( pConfig, "Internal", "HandlerLine" ) << endl;
	os << SECTION_P30_1;

	/*
	** Setup authentication 
	*/
#define SECTION_P60 "\n\
# ---\n\
#\n\
# Authenticate\n\
#\n\
#	Handle authentication according to realm\n\
#\n\
# ---\n\
<Object>\n\
	Name Authenticate\n\
	Class FlexibleHandlerClass\n\
	# Pass authentication if there is no realm \n\
	CheckAuth Condition=\"&not(&dblookup(response,string,AuthenticationRealm))\" \\\n\
\t\tReturnCode ReturnCode=COMPLETED\n"

	os << SECTION_P60;
	iTmp = 0;
	for(;; iTmp++ )
		{
		const char *pTmp = M_LookupValueEx( pConfig, "Realms", 
					"Basic", iTmp );
		if ( !pTmp ) { break; };

		os << "\tCheckAuth BasicAuth_" << pTmp << endl;
		};
	os << "</Object>" << endl;

	/*
	** Write information on the realms
	*/
#define SECTION_P61 "\n\
# ---\n\
#\n\
# Authentication realm objects\n\
#\n\
# ---\n"
	os << SECTION_P61;
	iTmp = 0;
	for(;; iTmp++ )
		{
		const char *pTmp = M_LookupValueEx( pConfig, "Realms", 
					"Basic", iTmp );
		if ( !pTmp ) { break; };

		os << "<Object>" << endl << "\tName BasicAuth_" << pTmp << endl;
		os << "\tClass BasicAuthClass" << endl; 
		/*
		** list all the users for this realm
		*/
		int iTmp2 = 0;
		PIString sRealm( "__" );
		sRealm.Concatenate( pTmp );
		os << "\tRealm \"" << pTmp << "\"" << endl;
		for(;; iTmp2++ )
			{
			const char *pTmp2 = M_LookupValueEx( pConfig, sRealm, 
					"Base64", iTmp2 );
			if ( !pTmp2 ) { break; };
			os << "\tBase64 \"" << pTmp2 << "\"" << endl;
			};

        os << "</Object>" << endl;
		};

    os << endl;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int Internal_writeDBValues( QConfig *pConfig, ostream &os )
{
    const char *pSectionName;
    for( int i=0; (pSectionName = M_LookupSection( pConfig, i )); i++ )
        {
        /*
        ** Dispatch before write callback, this function returns an
        ** integer value, non-zero for continue, zero for error/ignore.
        **
        ** This value is ignored right now. See below for after write 
        ** callback
        ** 
        */
        M_DispatchWriteCallBacks( pConfig, pSectionName,
            M_CB_BEFORESECTIONWRITE, os );
        
        os << '#' << SECTION_START << pSectionName << endl;
        const char *pVariable;
        const char *pValue;
        for( int j=0;
            ( M_LookupPair( pConfig, pSectionName, j, &pVariable, &pValue )); 
            j++
            )
            {
            os << '#' << pVariable << ':' << pValue << endl;
            };

        os << '#' << SECTION_END << endl;
        /*
        ** Dispatch after write callback
        **
        ** see notes above
        ** 
        */
        M_DispatchWriteCallBacks( pConfig, pSectionName,
            M_CB_AFTERSECTIONWRITE, os );
        };

    return 1;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int Pi3File_writeConfig( QConfig *pConfig, ostream &os )
{
	assert( pConfig );

    /*
    ** Set any defaults that have not already been set
    */
// LATER ---- this is temporary 0 until everything is ready.....
Internal_setupDefaults( pConfig );
    const char *pRetained = pConfig->GetRetained();

	/*
	** Write Pi3 language for the server
	*/
	os << "#!../bin/Pi3" << endl << endl << '#' << GENERATED_START << endl;
	Internal_writeCustomizedParagraphs( pConfig, os );
	os << '#' << GENERATED_END << endl;
	os << '#' << NOTMODIFIED_START << endl;

    if ( pRetained && *pRetained )
        {
        os << pRetained;
        }
    else
        {
        Internal_writeDefaultParagraphs( os ); 
        };

	os << '#' << NOTMODIFIED_END << endl;
	
    /*
    ** Write in the QConfig values
    */
    Internal_writeDBValues( pConfig, os );

	return 0;	/* success */
}


/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
static void Internal_newValue( QConfig *pConfig, const char *pSection,
    const char *pPair )
{
    assert( pPair );

    int i;
    for( i=0; pPair[i] && pPair[i]!=':'; i++ );
    PIString sVariable( pPair, i );
    PIString sValue;
    if ( pPair[i] )
        { sValue = &(pPair[i+1]); };
    pConfig->AddValue( pSection, sVariable, sValue );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int Pi3File_readConfig( QConfig *pConfig, istream &is )
{
    enum { BUF_SIZE=2047 };
	char szBuf[BUF_SIZE+1];
	
	PIString *pRetained = PI_NEW( PIString );
	
    PIString *pCurrentSection = 0;
	
    enum { NORMAL, SECTION, RETAINED } eState = NORMAL;
	
	for(; !is.eof();)
		{
		*szBuf = '\0';
		is.getline( szBuf, BUF_SIZE );
		szBuf[BUF_SIZE] = '\0';
	
		if ( eState!=RETAINED && ( *szBuf=='\0' || *szBuf!='#' ) )
			{ continue; };
	
		switch( eState )
			{
			case NORMAL:
				if ( !strncmp( &(szBuf[1]), SECTION_START, 
					sizeof( SECTION_START )-1 ) )
					{
                    PI_DELETE( pCurrentSection );
                    pCurrentSection = 0;
                    pCurrentSection = PI_NEW( PIString( 
                        &(szBuf[sizeof(SECTION_START)]) ) );
                    eState = SECTION;
                    }
				else if ( !strncmp( &(szBuf[1]), NOTMODIFIED_START, 
					sizeof( NOTMODIFIED_START )-1 ) )
					{ eState = RETAINED; };
				break;
	
			case SECTION:
				if ( !strncmp( &(szBuf[1]), SECTION_END, 
					sizeof( SECTION_END )-1 ) )
					{ eState = NORMAL; }
				else
                    {
                    Internal_newValue(pConfig, *pCurrentSection, &szBuf[1]);
                    };
				break;
					
			case RETAINED:
				if ( !strncmp( &(szBuf[1]), NOTMODIFIED_END, 
					sizeof( NOTMODIFIED_END )-1 ) )
					{ eState = NORMAL; }
				else
					{
					assert( pRetained );
					pRetained->Concatenate( szBuf );
					pRetained->Concatenate( '\n' );
					};
				break;
				
			default:
				assert( 0 );
			};		
		};
	
	/*
	** Done!
	*/
    pConfig->SetRetained( *pRetained );

	return 0;
}
	
