/*____________________________________________________________________________*\
 *

 Copyright (c) 1997 John Roy. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
	notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
	notice, this list of conditions and the following disclaimer in
	the documentation and/or other materials provided with the
	distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
	mentioning features must contain the following
	acknowledgment:
	"This product includes software developed by John Roy
	(http://www.johnroy.com/pi3/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: PropSheet.cpp$
 * $Date: Sun Aug 10 06:36:50 1997$
 *
 Description:

\*____________________________________________________________________________*/
//$SourceTop:$

#include <iostream.h>
#include <stdio.h>

#include "IFaceBse.h"
#include "PIStrStr.h"
#include "StrToken.h"
#include "QConfig.h"
#include "Admin.h"
#include "DeQuote.h"

/*
** #define D { cerr << __FILE__ << ": " << __LINE__ << endl; }
*/
#define D

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
#define KEY_CONF_TITLE                      "Title"
#define KEY_CONF_PROPERTYPAGE               "PropertyPage"
#define KEY_CONF_FILENAME                   "FileName"
#define KEY_CONF_HOST                       "Host"
#define KEY_CONF_FLAG                       "Flag"
#define KEY_CONF_FLAG_CREATEDEFAULT         "CreateDefault"
#define KEY_CONF_FLAG_LOADFILE              "LoadFile"

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
#if 0
	/*
	** HTML documentation for this handler
	*/
/*___+++HTMLDOC_BEGIN+++___*/
Name:
    PropSheet

Description:
    Generates a property sheet window.

Example:
	<PRE>
	&ltObject&gt
		Name PropSheet
		Class PropSheetClass
        PropertyPage GeneralPage
	&lt/Object&gt

	</PRE>

/*___+++HTMLDOC_END+++___*/
#endif

/*____________________________________________________________________________*\
 *
 Class:
 Description:
\*____________________________________________________________________________*/
class PropSheet : public IFaceBase
{
private:
    Admin *pAdmin;
    PIString sTitle;
    DblList lPages;
    Interface tInterface;
    int iCreateDefault;
    PIString sFileName;
    PIString sHost;
    QConfig *pConfig;

protected:

	int Parameter( const char *pVariable, const char *pValue,
		const char *pWhere )
		{
		assert( pVariable && pValue );

		PIOStrStream os;

		os << pWhere << "PropSheet: ";

        if ( !PIUtil_stricmp( pVariable, KEY_CONF_TITLE ) )
			{
            sTitle = pValue;
            }
        else if ( !PIUtil_stricmp( pVariable, KEY_CONF_PROPERTYPAGE ) )
			{
            /*
            ** Attempt to load the object
            */
            PIObject *pPageObject = PIObject_load(
                PIObject_getDB( Object() ), 0, pValue, 0, 0 );

            if ( !pPageObject )
                {
                /* --- error message already set --- */
                return 0;
                };

            Interface *pInterface = &tInterface;

            pInterface->iMessage = INTRFACE_MSG_INIT;
            if ( PILogic_execute( pPageObject, 1, (const char **)&pInterface )
                != PIAPI_COMPLETED ) 
                {
                os << "Failed to initialize property sheet '" <<
                    PIObject_getName( pPageObject ) << "'." << ends;
                PIObject_delete( pPageObject, 0, 0 );
			    CONFIG_ERR( Object(), os.str() );
			    return 0;
                };

            if ( iCreateDefault )
                {
                pInterface->iMessage = INTRFACE_MSG_DEFAULTS;
                if ( PILogic_execute( pPageObject, 1,
                    (const char **)&pInterface )
                    != PIAPI_COMPLETED ) 
                    {
                    os << "Failed to setup defaults for property sheet '" <<
                        PIObject_getName( pPageObject ) << "'." << ends;
                    PIObject_delete( pPageObject, 0, 0 );
			        CONFIG_ERR( Object(), os.str() );
			        return 0;
                    };
                };

            /*
            ** Page created
            */
            lPages.Append( (DblList::type)pPageObject );
			}
        else if ( !PIUtil_stricmp( pVariable, KEY_CONF_FLAG ) )
			{
            if ( !PIUtil_stricmp( pValue, KEY_CONF_FLAG_CREATEDEFAULT ) )
                {
                iCreateDefault = 1;
                }
            else if ( !PIUtil_stricmp( pValue, KEY_CONF_FLAG_LOADFILE ) )
                {
                PIString sError;
                if ( Admin_loadConfig( pAdmin, pConfig, sError ) )
                    {
                    if ( sError.Len()>0 )
                        {
                        os << (const char *)sError << ends;
                        }
                    else
                        {
                        os << "Failed to load configuration from server."
                            << ends;
                        };
			        CONFIG_ERR( Object(), os.str() );
			        return 0;
                    };
#if 0
                if ( Commands_loadConfigFromFile( pConfig, sFileName ) )
                    {
			        os << "Failed to load file '" << sFileName <<
				        "'" << ends;
			        CONFIG_ERR( Object(), os.str() );
			        return 0;
                    };
#endif
                }
            else
                {
			    os << "Unknown flag '" << pValue <<
				    "'" << ends;
			    CONFIG_ERR( Object(), os.str() );
			    return 0;
                };
            }
        else if ( !PIUtil_stricmp( pVariable, KEY_CONF_FILENAME ) )
			{
            sFileName = pValue;
            }
        else if ( !PIUtil_stricmp( pVariable, KEY_CONF_HOST ) )
			{
            /* --- host is read below --- */
            }
		else
			{
			os << "Unknown directive '" << pVariable <<
				"'" << ends;
			CONFIG_ERR( Object(), os.str() );
			return 0;
			};

		return 1;
		};

public:
	PropSheet( PIObject *pObject, int iArgc, const char *ppArgv[] )
	:	IFaceBase( pObject ),
        pAdmin( 0 ),
        sTitle( "Pi3 Property Sheet" ),
        iCreateDefault( 0 ),
        pConfig( CreateQConfig() )
		{
        /*
        ** Read host - LATER or file - also LATER - check
        */
        const char *pHost = (const char *)PIConfig_lookupValue(
            PIObject_getConfigurationDB( Object() ), KEY_CONF_HOST, 0, 0 );
        
        DeQuote tHost( pHost );
        sHost = (const PIString &)tHost;

        pAdmin = Admin_newRemote( Object(), sHost, 80, "", "" );

        PIDB_add( PIObject_getDB( Object() ), PIDBTYPE_OPAQUE,
            QCONFIG_KEY, pConfig, 0 );

        ReadParameters( iArgc, ppArgv );

		if ( !IsOK() )
            { return; };

		};

	~PropSheet()
		{
        PIString sError;
        int iRet = PIAPI_COMPLETED;
        iRet = Admin_saveConfig( pAdmin, pConfig, sError );
if ( iRet )
{
if ( sError.Len() )
    {
    MessageBox(NULL, (const char *)sError, "Error", MB_OK );
    }
else
    {
    MessageBox(NULL, "Failed to save configuration", "Error", MB_OK );
    };
}
        DestroyQConfig( pConfig );
        Admin_delete( pAdmin );
        for( DblListIterator i( lPages ); !i.BadIndex(); i++ )
            {
            PIObject_delete( (PIObject *)i.Current(), 0, 0 );
            };
		};

    virtual int Handle( Interface &tWindow )
        {
        switch( tWindow.iMessage )
            {
            case INTRFACE_MSG_INIT:
                break;

            case INTRFACE_MSG_WIN32CREATE:
                return Win32Create( tWindow );

            case INTRFACE_MSG_LOAD:
            case INTRFACE_MSG_DEFAULTS:
                return PIAPI_ABORT;

            default:
                assert( 0 );
                return PIAPI_ABORT;
            };

        return PIAPI_COMPLETED;
        };

    int Win32Create( Interface &tWindow )
		{
        /*
        ** Create the property sheet
        */
        int iNumPages = lPages.Size();

        if ( !iNumPages )
            {
            tWindow.sError = "PropSheet: Cannot create property sheet \
with no property pages.";
            return PIAPI_ERROR;
            };

        HPROPSHEETPAGE *pSheets = PI_NEW( HPROPSHEETPAGE[iNumPages] );
        
        int j=0;
        for( DblListIterator i( lPages ); !i.BadIndex(); i++)
            {
            PIObject *pPageObject = (PIObject *)i.Current();
            PROPSHEETPAGE *pPsp = (PROPSHEETPAGE *)PIDB_lookup(
                PIObject_getDB( pPageObject ), PIDBTYPE_OPAQUE, 
                INTRFACE_PROPSHEETPAGE, 0 );
            HPROPSHEETPAGE hPropSheetPage = ::CreatePropertySheetPage( pPsp );
// MessageBox( NULL, pPsp->pszTemplate, "foo1", MB_OK );
                    
            if ( !hPropSheetPage )
                {
                return PIAPI_ERROR;
                };

            pSheets[j++] = hPropSheetPage;
            };
        
        /*
        ** setup property structure
        */
        PROPSHEETHEADER tPsh;
        memset( &tPsh, 0, sizeof( PROPSHEETHEADER ) );
        tPsh.dwSize = sizeof( PROPSHEETHEADER );
//        tPsh.dwFlags = PSH_MODELESS | PSH_DEFAULT ;
        tPsh.dwFlags = PSH_DEFAULT ;
        tPsh.hInstance = NULL;
        tPsh.hwndParent = NULL;
        tPsh.pszIcon = NULL;
        tPsh.nPages = iNumPages;
        tPsh.phpage = pSheets;
        tPsh.pszCaption = sTitle;
    
        /*
        ** Create modeless property sheet
        */
        HWND hPropSheet = (HWND)PropertySheet( &tPsh );

        PI_DELETE( [] pSheets );

        if ( !hPropSheet )
            {
            tWindow.sError = "PropSheet: Failed to create property sheet.";
            return PIAPI_ERROR;
            };

        /*
        ** Done
        */
        return PIAPI_COMPLETED;
		};
};

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int PropSheet_constructor( PIObject *pObj,
	int iArgc, const char *ppArgv[] )
{
	return IFaceBase_constructor( pObj, PI_NEW( PropSheet( pObj,
		iArgc, ppArgv ) ) );
}

#if 0
/*___+++CNF_BEGIN+++___*/
	<Class>
		Name PropSheetClass
		Type LogicExtension
		Library IntrFace
		OnClassLoad IFaceBase_onClassLoad
		Constructor PropSheet_constructor
		CopyConstructor IFaceBase_copyConstructor
		Destructor IFaceBase_destructor
		Execute IFaceBase_execute
	</Class>

	<Object>
		Name PropSheet
		Class PropSheetClass
	</Object>

/*___+++CNF_END+++___*/
#endif

