/*____________________________________________________________________________*\
 *

 Copyright (c) 1997 John Roy. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
	notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
	notice, this list of conditions and the following disclaimer in
	the documentation and/or other materials provided with the
	distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
	mentioning features must contain the following
	acknowledgment:
	"This product includes software developed by John Roy
	(http://www.johnroy.com/pi3/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: MIMEType.cpp$
 * $Date: Sun Aug 10 06:36:50 1997$
 *
 Description:

\*____________________________________________________________________________*/
//$SourceTop:$

#include <iostream.h>
#include <stdio.h>
#include <windowsx.h>

#include "IFaceBse.h"
#include "PIStrStr.h"
#include "StrToken.h"

#include "QConfig.h"
#include "VServers.hrc"
#include "MiscUtil.h"

/*
** #define D { cerr << __FILE__ << ": " << __LINE__ << endl; }
*/
#define D

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
#define C_IPRESOURCENAME                "IPVServersPage"
#define C_NONIPRESOURCENAME             "NonIPVServersPage"

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
#define IPSECTION       "IPVirtualServers"
#define NONIPSECTION    "NonIPVirtualServers"
#define VARIABLE        "Server"
#define KEY_CONF_FLAG   "Flag"
#define FLAG_IP         "IP"
#define FLAG_NONIP      "NonIP"

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
#if 0
	/*
	** HTML documentation for this handler
	*/
/*___+++HTMLDOC_BEGIN+++___*/
Name:
    VServersPage

Description:
    No description yet.

Example:
	<PRE>
	&ltObject&gt
		Name VServersPage
		Class VServersPageClass
        PropertyPage VServersPage
	&lt/Object&gt

	</PRE>

/*___+++HTMLDOC_END+++___*/
#endif

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
static BOOL CALLBACK fnDialogProc(HWND hDlg, UINT uMsg, WPARAM wParam,
    LPARAM lParam);

/*____________________________________________________________________________*\
 *
 Class:
 Description:
\*____________________________________________________________________________*/
class VServersPage : public IFaceBase
{
private:
    QConfig *pConfig;
    PROPSHEETPAGE tPsp;
    Interface *pInterface;
    PIString sInstallDir;       /* Server root of parent virtual server */

    /*
    ** window private values
    */
    HWND hWndList;
    HWND hWndHost;
    HWND hWndServerRoot;
    HWND hWndAdmin;
    HWND hWndPassword;
    HWND hWndConfirm;
    HWND hWndAdd;
    HWND hWndUpdate;
    HWND hWndCopy;
    HWND hWndDelete;
    HWND hWndMonitorCaption;
    HWND hWndMonitor;
    DblList lHosts;
    DblList lServerRoots;
    DblList lAdmins;
    DblList lPasswords;

    /*
    ** Difference between IP and Non-IP virtual servers
    */
    int iIsIP;
    PIString sSection;
    PIString sResourceName;

	int Parameter( const char *pVariable, const char *pValue,
		const char *pWhere )
		{
		assert( pVariable && pValue );

		PIOStrStream os;

		os << pWhere << "VServersPage: ";

        if ( !PIUtil_stricmp( KEY_CONF_FLAG, pVariable ) )
            {
            if ( !PIUtil_stricmp( FLAG_IP, pValue ) )
                {
                iIsIP = 1;
                sSection = IPSECTION;
                sResourceName = C_IPRESOURCENAME;
                }
            else if ( !PIUtil_stricmp( FLAG_NONIP, pValue ) )
                {
                iIsIP = 0;
                sSection = NONIPSECTION;
                sResourceName = C_NONIPRESOURCENAME;
                }
            else
                {
			    os << "Unknown directive '" << pVariable <<
				    "'" << ends;
			    CONFIG_ERR( Object(), os.str() );
			    return 0;
                };
            }
		else
			{
			os << "Unknown directive '" << pVariable <<
				"'" << ends;
			CONFIG_ERR( Object(), os.str() );
			return 0;
			};

		return 1;
		};

public:
	VServersPage( PIObject *pObject, int iArgc, const char *ppArgv[] )
	:	IFaceBase( pObject ),
        pConfig( FindParentQConfig( PIObject_getDB( pObject ) ) ),
        hWndList( 0 ),
        hWndHost( 0 ),
        hWndServerRoot( 0 ),
        hWndAdmin( 0 ),
        hWndPassword( 0 ),
        hWndConfirm( 0 ),
        hWndAdd( 0 ),
        hWndUpdate( 0 ),
        hWndCopy( 0 ),
        hWndDelete( 0 ),
        hWndMonitorCaption( 0 ),
        hWndMonitor( 0 ),
        pInterface( 0 ),
        iIsIP( -1 )
		{
        /*
        ** Read input parameters
        */
		ReadParameters( iArgc, ppArgv );

        if ( !IsOK() )
            { return; };

        if ( iIsIP==-1 )
            {
            CONFIG_ERR( Object(), "VServersPage: One of the values 'IP' or \
'NonIP' must be specified for 'Flag'." );
            SetOK( 0 );
            return;
            };

        /*
        ** Setup the PROPSHEETPAGE
        */
        memset( &tPsp, 0, sizeof( PROPSHEETPAGE ) );
        tPsp.dwSize = sizeof( PROPSHEETPAGE );
        tPsp.dwFlags = PSP_DEFAULT ;
        tPsp.hInstance = PIDLL_getSystemHandle( PIClass_getLibrary(
            PIObject_getClass( Object() ) ) );

        /* Status page */
        tPsp.pszTemplate = sResourceName;
        tPsp.pfnDlgProc = (DLGPROC)fnDialogProc;
        tPsp.lParam = (LPARAM)this;

        /*
        ** Set the install directory 
        */
        sInstallDir = M_LookupValue( pConfig, "General", "InstallDir" );

        /*
        ** Set the data DB of this object
        */
        PIDB *pDB = PIObject_getDB( Object() );
        PIDB_add( pDB, PIDBTYPE_OPAQUE, INTRFACE_WINDOWTYPE,
            (void *)INTRFACE_PROPPAGE, 0 );
        PIDB_add( pDB, PIDBTYPE_OPAQUE, INTRFACE_PROPSHEETPAGE,
            (void *)&tPsp, 0 );
        
        /*
        ** Done
        */
		};

	~VServersPage()
		{
		};

    virtual int Handle( Interface &tWindow )
        {
        switch( tWindow.iMessage )
            {
            case INTRFACE_MSG_INIT:
                pInterface = &tWindow;
                M_RegisterCallBack(pConfig, sSection, CallBack, this );
                break;

            case INTRFACE_MSG_WIN32CREATE:
                return Win32Create( tWindow );

            case INTRFACE_MSG_LOAD:
                return PIAPI_ABORT;

            case INTRFACE_MSG_DEFAULTS:
                break;

            default:
                assert( 0 );
                return PIAPI_ABORT;
            };

        return PIAPI_COMPLETED;
        };

    int Win32Create( Interface &tWindow )
        {
        /* --- property sheet pages should not be created --- */
        return PIAPI_ABORT;
		};

    int Internal_verifyFields( HWND hDlg, char *pszBuf, int iBufSize )
        {
	    int iToLen;
        PIString sPassword;
	
	    /*
	    ** Get the 'ServerRoot' field
	    */
	    ::SendMessage( hWndServerRoot, WM_GETTEXT, iBufSize, (LPARAM)pszBuf );
	
	    /*
	    ** 'ServerRoot' field must end with '/'
	    */
	    assert( *pszBuf );
	    iToLen = strlen(pszBuf);
	    if ( pszBuf[iToLen-1]!='/' && pszBuf[iToLen-1]!='\\' )
	        {
	        ::MessageBox( hDlg, "'ServerRoot' field must end with a \
forward-slash ('/') or a backslash ('\\').",
	            "Error in 'ServerRoot' Field", MB_OK | MB_ICONEXCLAMATION );
	
	        ::SetFocus( hWndServerRoot );
	        ::SendMessage( hWndServerRoot, EM_SETSEL, (WPARAM)(INT)0,
                (LPARAM)(INT)-1 );
	
	        goto verify_fail;
	        };

        /*
        ** Check that the password field matches the confirm field
        */
	    ::SendMessage( hWndPassword, WM_GETTEXT, iBufSize, (LPARAM)pszBuf );
        sPassword = pszBuf;
	    ::SendMessage( hWndConfirm, WM_GETTEXT, iBufSize, (LPARAM)pszBuf );
        if ( PIUtil_stricmp( sPassword, pszBuf ) )
            {
	        ::MessageBox( hDlg, "'Password' field does not match \
'Confirm' field.",
	            "Error in Password", MB_OK | MB_ICONEXCLAMATION );
	
	        ::SetFocus( hWndPassword );
	        ::SendMessage( hWndPassword, EM_SETSEL, (WPARAM)(INT)0,
                (LPARAM)(INT)-1 );
	
	        goto verify_fail;
            };
	
	    /*
	    ** Good
	    */
	    return 1;
	
verify_fail:
	    return 0;
        };
	
    void Internal_splitLine( const char *pLine, PIString &sItem1,
        PIString &sItem2, PIString &sItem3, PIString &sItem4 )
        {
        assert( pLine );

		int i, j;
	    i = 0;
		for( ; pLine[i] && isspace(pLine[i]); i++ );
	    j = i;
		for( ; pLine[i] && !isspace(pLine[i]); i++ );
		
        PIString sTmp1( &(pLine[j]), i-j );
        sItem1 = sTmp1;
	
        /* --- scan over whitespace to start of second field --- */
        for( ; pLine[i] && isspace(pLine[i]); i++ );
        j = i;
        for( ; pLine[i] && !isspace(pLine[i]); i++ );

        PIString sTmp2( &(pLine[j]), i-j );
        sItem2 = sTmp2;
	
		/* --- scan over whitespace to start of second field --- */
		for( ; pLine[i] && isspace(pLine[i]); i++ );
	    j = i;
		for( ; pLine[i] && !isspace(pLine[i]); i++ );

        PIString sTmp3( &(pLine[j]), i-j );
        sItem3 = sTmp3;
	
		/* --- scan over whitespace to start of third field --- */
		for( ; pLine[i] && isspace(pLine[i]); i++ );
	    j = i;
		for( ; pLine[i] && !isspace(pLine[i]); i++ );
	
        PIString sTmp4( &(pLine[j]), i-j );
        sItem4 = sTmp4;
        };

    void Internal_addLine( const char *pLine )
        {
		assert( pLine );
        assert( lHosts.Size()==lServerRoots.Size() );
		assert( lServerRoots.Size()==lAdmins.Size() );
		assert( lAdmins.Size()==lPasswords.Size() );
	
        PIString sHost;
        PIString sServerRoot;
        PIString sAdmin;
        PIString sPassword;

        Internal_splitLine( pLine, sHost, sServerRoot, sAdmin, sPassword );

        lHosts.Append( (DblList::type)PI_NEW( PIString( sHost )));
		lServerRoots.Append((DblList::type)PI_NEW(PIString( sServerRoot )));	
		lAdmins.Append((DblList::type)PI_NEW( PIString( sAdmin )));	
		lPasswords.Append((DblList::type)PI_NEW( PIString( sPassword )));	
	
        assert( lHosts.Size()==lServerRoots.Size() );
		assert( lServerRoots.Size()==lAdmins.Size() );
		assert( lAdmins.Size()==lPasswords.Size() );
        };

    void Internal_clearArrays()
        {
		DblListIterator w( lHosts );
		DblListIterator x( lServerRoots );
		DblListIterator y( lAdmins );
		DblListIterator z( lPasswords );
		for( ; !x.BadIndex(); )
			{
			assert( !w.BadIndex() && !x.BadIndex() && !y.BadIndex()
                && !z.BadIndex() );
            PI_DELETE( (PIString *)w.Current() );
			PI_DELETE( (PIString *)x.Current() );
			PI_DELETE( (PIString *)y.Current() );
			PI_DELETE( (PIString *)z.Current() );

            w++;
            x++;
            y++;
            z++;
			};
		lHosts.Clear();
		lServerRoots.Clear();
		lAdmins.Clear();
		lPasswords.Clear();
        };
	
    void Internal_reloadArrays()
        {
		enum { BUF_SIZE=2047 };
		char szBuf[BUF_SIZE+1];
	
		Internal_clearArrays();
	
		/*
		** Get number of items
		*/
		int iCount = ::SendMessage( hWndList, LB_GETCOUNT, 0, 0 );
	
		/* 
		** Insert from listbox into internal arrays
		*/
		int i;
		for( i=0; i<iCount; i++ )
			{
			::SendMessage( hWndList, LB_GETTEXT, i, (LPARAM)szBuf );
			Internal_addLine( szBuf );
			};
        };
	
    void Internal_updateButtons( WPARAM wParam, LPARAM lParam,
	    HWND hWndEdit1, HWND hWndEdit2, const char *pData )
        {
        enum { BUF_SIZE=511 };
        char szB[BUF_SIZE+1];
        const char *pCaption = 0;

        UINT uId = GET_WM_COMMAND_ID(wParam, lParam);
        switch( GET_WM_COMMAND_CMD( wParam, lParam ) )
            {
            case LBN_SETFOCUS:
            case LBN_SELCHANGE:
            case EN_SETFOCUS:
            case EN_CHANGE:
                *szB = '\0'; szB[BUF_SIZE] = '\0';
                switch( uId )
                    {
                    case IDC_HOST: 
                        ::SendMessage(hWndHost,WM_GETTEXT,BUF_SIZE,(LPARAM)szB);
                        pCaption = "IP Address";
                        break;

                    case IDC_SERVERROOT: 
                        {
                        PIString sTmp;
                        ::SendMessage(hWndServerRoot,WM_GETTEXT,BUF_SIZE,
                            (LPARAM)szB);
                        RelativeToAbsolutePath( sInstallDir, szB, sTmp );
                        strncpy( szB, sTmp, BUF_SIZE );
                        szB[BUF_SIZE] = '\0';
                        pCaption = "Server Root";
                        };
                        break;

                    case IDC_ADMINISTRATOR: 
                        ::SendMessage(hWndAdmin,WM_GETTEXT,BUF_SIZE,
                            (LPARAM)szB);
                        pCaption = "Administrator's Username";
                        break;

                    case IDC_PASSWORD: 
                        *szB = '\0';
                        pCaption = "Administrator's Password";
                        break; 

                    case IDC_CONFIRM: 
                        *szB = '\0';
                        pCaption = "Confirm Administrator's Password";
                        break;

                    case IDC_LIST:
                        pCaption = ""; *szB = '\0'; break;

                    default:
                        assert( 0 );
                    };
                assert( pCaption );
                ::SendMessage( hWndMonitorCaption, WM_SETTEXT, 0,
                    (LPARAM)pCaption);
                ::SendMessage( hWndMonitor, WM_SETTEXT, 0, (LPARAM)szB);
                break;
                
            case EN_KILLFOCUS:
                ::SendMessage(hWndMonitorCaption, WM_SETTEXT, 0, (LPARAM)"");
                ::SendMessage(hWndMonitor, WM_SETTEXT, 0, (LPARAM)"");
                break;

            default:
                return;
            };

        /*
        ** Continue if EN_CHANGE
        */
        if ( GET_WM_COMMAND_CMD( wParam, lParam )!=EN_CHANGE )
            { return; };
	
		int iExistingIndex = -1;
		assert( pData );
		int iChangeToIndex = -1;
		int iIndex = 0;
		if ( *pData )
			{
			for( DblListIterator i( lHosts );!i.BadIndex(); i++,iIndex++)
				{
				PIString *pString = (PIString *)i.Current();
				assert( pString );
				if ( !PIUtil_strncmpi( (*pString), pData, strlen( pData ) ) )
					{
					iChangeToIndex = iIndex;
					/*
					** If this is a full index, select it
					*/
					if ( !stricmp( (*pString), pData ) )
						{ iExistingIndex = iIndex; };
					break;
					};
				};
			};
	
		/*
		** Remove listbox selection or scroll to correct line
		** as appropriate 
		*/
		::SendMessage( hWndList, LB_SETCURSEL, iChangeToIndex, 0 );
	
		/*
		** Set buttons
		*/
		if ( ::SendMessage( hWndEdit1, EM_LINELENGTH, 0, 0 )==0 ||
		    ::SendMessage( hWndServerRoot, EM_LINELENGTH, 0, 0 )==0 ||
		    ::SendMessage( hWndAdmin, EM_LINELENGTH, 0, 0 )==0 ||
		    ::SendMessage( hWndPassword, EM_LINELENGTH, 0, 0 )==0 ||
		    ::SendMessage( hWndConfirm, EM_LINELENGTH, 0, 0 )==0 )
			{
			if ( ::IsWindowEnabled( hWndAdd ) )
                ::EnableWindow( hWndAdd, FALSE );
			if ( ::IsWindowEnabled( hWndUpdate ) )
                ::EnableWindow( hWndUpdate, FALSE );
			}
		else if ( SendMessage( hWndEdit2, EM_LINELENGTH, 0, 0 )!=0 )
			{
			if ( iExistingIndex>-1 )/* --- disable add, enable replace --- */
				{
				if ( ::IsWindowEnabled( hWndAdd ) )
                    ::EnableWindow( hWndAdd, FALSE );
				if ( !::IsWindowEnabled( hWndUpdate ) )
                    ::EnableWindow( hWndUpdate, TRUE );
				}
			else				/* --- disable replace, enable add --- */
				{
				if ( !::IsWindowEnabled( hWndAdd ) )
                    ::EnableWindow( hWndAdd, TRUE );
				if ( IsWindowEnabled( hWndUpdate ) )
                    ::EnableWindow( hWndUpdate, FALSE );
				};
			};
	
		/*
		** If there is a selected item in the listbox then
		** enable copy & delete, otherwise disable copy & delete	
		*/
		if ( SendMessage( hWndList, LB_GETCURSEL, 0, 0 )==LB_ERR )
			{
            ::EnableWindow( hWndCopy, FALSE );
            ::EnableWindow( hWndDelete, FALSE );
            }
		else
	        {
/*
** Copy not implemented right now
            ::EnableWindow( hWndCopy, TRUE );
*/
            ::EnableWindow( hWndDelete, TRUE );
            };
        };
    
    int fnCallBack( const char *pSection, const char *pVariable,
        int iAction, SectionWriteCallBackStructure *pStruct )
        {
        int iTmp;
        const char *pTmp;
        switch( iAction )
            {
            case M_CB_BEFORESECTIONWRITE:
				/*
				** Save data
				*/
				iTmp = 0;
				for(;; iTmp++ )
					{
					pTmp = M_LookupValueEx( pConfig, sSection, 
						VARIABLE, iTmp );
					if ( !pTmp )
						{ break; };
    
                    PIString sHost;
                    PIString sServerRoot;
                    PIString sAdmin;
                    PIString sPassword;
	
                    Internal_splitLine( pTmp, sHost, sServerRoot,
                        sAdmin, sPassword );

                    pStruct->os << "<Object>" << endl;
                    pStruct->os << "\tName " << sSection << "_" << sHost
                        << endl;
                    pStruct->os << "\tClass SiteRootClass" << endl;
                    pStruct->os << "\tRootDirectory \"" << sServerRoot
                        << "\"" << endl;
                    pStruct->os << "\tAdminUser \"" << sAdmin << "\"" << endl;
                    pStruct->os << "\tAdminPassword \"" << sPassword
                        << "\"" << endl;
                    pStruct->os << "\tLogicObject \"Default\"" << endl;
                    pStruct->os << "\tConfigFile \"" << sServerRoot
                        << "Config.pi3\"" << endl;
                    pStruct->os << "</Object>" << endl;
					};
                break;
 
            default:;
            };
        return 1;
        };

    /*
    ** Callback function for notification that the configuration
    ** file is being written
    */
    static int CallBack( const char *pSection, const char *pVariable,
        int iAction, void *pCallBackData, void *pData )
        {
        return ((VServersPage *)pData)->fnCallBack( pSection, pVariable,
            iAction, (SectionWriteCallBackStructure *)pCallBackData );
        };
	
    BOOL CALLBACK DialogProc(HWND hDlg, UINT uMsg, WPARAM wParam,
		LPARAM lParam)
	    {
		enum { BUF_SIZE=2047 };
		char szBuf[BUF_SIZE+1];
		int aTabstop[4];
	
		switch (uMsg)
			{
			case WM_INITDIALOG:
				HourglassOn();
				{
				const char *pTmp;
				int iTmp;
				hWndList = GetDlgItem( hDlg, IDC_LIST );
				hWndHost = GetDlgItem( hDlg, IDC_HOST );
				hWndServerRoot= GetDlgItem( hDlg, IDC_SERVERROOT );
				hWndAdmin = GetDlgItem( hDlg, IDC_ADMINISTRATOR );
				hWndPassword = GetDlgItem( hDlg, IDC_PASSWORD );
				hWndConfirm = GetDlgItem( hDlg, IDC_CONFIRM );
				hWndAdd = GetDlgItem( hDlg, IDC_ADD );
				hWndUpdate = GetDlgItem( hDlg, IDC_UPDATE );
				hWndCopy = GetDlgItem( hDlg, IDC_COPY );
				hWndDelete= GetDlgItem( hDlg, IDC_DELETE );
				hWndMonitorCaption = GetDlgItem( hDlg, IDC_MONITORCAPTION );
				hWndMonitor = GetDlgItem( hDlg, IDC_MONITOR );
	
				assert( hWndList && hWndHost && hWndServerRoot );
				assert( hWndAdmin && hWndPassword && hWndConfirm );
				assert( hWndAdd && hWndUpdate && hWndCopy );
				assert( hWndDelete && hWndMonitorCaption && hWndMonitor );
	
				::EnableWindow( hWndAdd, FALSE );
				::EnableWindow( hWndUpdate, FALSE );
				::EnableWindow( hWndCopy, FALSE );
				::EnableWindow( hWndDelete, FALSE );
	
				/*
				** Limit edit field lengths, really just a sanity thing
				*/
				::SendMessage( hWndHost, EM_SETLIMITTEXT, 256, 0 );
				::SendMessage( hWndServerRoot, EM_SETLIMITTEXT, 256, 0 );
				::SendMessage( hWndAdmin, EM_SETLIMITTEXT, 256, 0 );
				::SendMessage( hWndPassword, EM_SETLIMITTEXT, 256, 0 );
				::SendMessage( hWndConfirm, EM_SETLIMITTEXT, 256, 0 );
	
				/*
				** Setup tabstops
				*/
				aTabstop[0] = 0;
				aTabstop[1] = 60;
				aTabstop[2] = 120;
				aTabstop[3] = 2000;
				SendMessage( hWndList, LB_SETTABSTOPS, 4, (LPARAM)&aTabstop );
	
				/*
				** Load all data into the listbox
				*/
				iTmp = 0;
				for(;; iTmp++ )
					{
					pTmp = M_LookupValueEx( pConfig, sSection, 
						VARIABLE, iTmp );
					if ( !pTmp )
						{ break; };
	
					::SendMessage( hWndList, LB_ADDSTRING, 0, (LPARAM)pTmp );
					};
				Internal_reloadArrays();
				}
				HourglassOff();
				break;
	
			case WM_DESTROY:
				{
				Internal_clearArrays();
				};
				break;
	
	        case WM_COMMAND:
				switch (GET_WM_COMMAND_ID(wParam, lParam))
					{
					case IDC_ADD:
	                    {
	                    /*
	                    ** Verify fields are valid
	                    */
	                    if ( !Internal_verifyFields( hDlg, szBuf, BUF_SIZE ) )
	                        { break; };
	
						int i, j;
						*szBuf = '\0';
                        i = 0;
	
						::SendMessage( hWndHost, WM_GETTEXT, BUF_SIZE,
	                        (LPARAM)szBuf );
						for( i=0; i<BUF_SIZE && szBuf[i] && szBuf[i]!='\t';
                            i++ );
						if ( szBuf[i] ) { szBuf[i] = '\0'; };
						lHosts.Append( (DblList::type)PI_NEW(
                            PIString(szBuf )));
				        szBuf[i++] = '\t';
	
						::SendMessage( hWndServerRoot, WM_GETTEXT, BUF_SIZE-i,
	                        (LPARAM)&(szBuf[i]) );
						for(j=i;i<BUF_SIZE && szBuf[i] && szBuf[i]!='\t';i++);
						if ( szBuf[i] ) { szBuf[i] = '\0'; };
						lServerRoots.Append( (DblList::type)PI_NEW(
	                        PIString( &(szBuf[j]) ) ) );
						szBuf[i++] = '\t';
	
						::SendMessage( hWndAdmin, WM_GETTEXT, BUF_SIZE-i,
	                        (LPARAM)&(szBuf[i]) );
						for(j=i;i<BUF_SIZE && szBuf[i] && szBuf[i]!='\t';i++);
						if ( szBuf[i] ) { szBuf[i] = '\0'; };
						lAdmins.Append( (DblList::type)PI_NEW(
	                        PIString( &(szBuf[j]) ) ) );
						szBuf[i++] = '\t';
	
						::SendMessage( hWndPassword, WM_GETTEXT, BUF_SIZE-i,
	                        (LPARAM)&(szBuf[i]) );
						for(j=i; i<BUF_SIZE && szBuf[i] && szBuf[i]!='\t'; i++);
						if ( szBuf[i] ) { szBuf[i] = '\0'; };
						lPasswords.Append( (DblList::type)PI_NEW(
	                        PIString( &(szBuf[j]) ) ) );
	
						::SendMessage(hWndList, LB_ADDSTRING,0,(LPARAM)szBuf );
	
						/*
						** Clear edit fields, focus to list
						*/
                        ::SendMessage(hWndHost, WM_SETTEXT, 0,(LPARAM)"");
						::SendMessage(hWndServerRoot, WM_SETTEXT, 0,(LPARAM)"");
						::SendMessage(hWndAdmin, WM_SETTEXT, 0, (LPARAM)"" );
						::SendMessage(hWndPassword, WM_SETTEXT, 0, (LPARAM)"" );
						::SendMessage(hWndConfirm, WM_SETTEXT, 0, (LPARAM)"" );
						Internal_reloadArrays();
						int iIndex = ::SendMessage( hWndList,
                            LB_FINDSTRINGEXACT, 0, (LPARAM)szBuf );
						if ( iIndex!=LB_ERR )
                            ::SendMessage(hWndList, LB_SETTOPINDEX, iIndex, 0);
						::SetFocus( hWndList );
						PropSheet_Changed( ::GetParent( hDlg ), hDlg );
						};
						break;
		
					case IDC_UPDATE:
	                    {
	                    /*
	                    ** Verify fields are valid
	                    */
	                    if ( !Internal_verifyFields( hDlg, szBuf, BUF_SIZE ) )
	                        { break; };
	
						/*
						** Get selected item index 
						*/
						int iCurSel = ::SendMessage( hWndList, LB_GETCURSEL,
                            0, 0 );
	
						/*
						** Get text to add and add it to internal lists only
						*/
						int i, j;
						*szBuf = '\0';
                        i = 0;
	
                        ::SendMessage( hWndHost, WM_GETTEXT, BUF_SIZE,
                            (LPARAM)szBuf );
                        for( i=0; i<BUF_SIZE && szBuf[i] && szBuf[i]!='\t';i++);
                        if ( szBuf[i] ) { szBuf[i] = '\0'; };
                            lHosts.Append( (DblList::type)PI_NEW(
                                PIString(szBuf )));
                        szBuf[i++] = '\t';
	
						::SendMessage( hWndServerRoot, WM_GETTEXT, BUF_SIZE-i,
	                        (LPARAM)&(szBuf[i]) );
						for(j=i;i<BUF_SIZE && szBuf[i] && szBuf[i]!='\t';i++);
						if ( szBuf[i] ) { szBuf[i] = '\0'; };
						lServerRoots.Append( (DblList::type)PI_NEW(
	                        PIString( &(szBuf[j]) ) ) );
						szBuf[i++] = '\t';
	
						::SendMessage( hWndAdmin, WM_GETTEXT, BUF_SIZE-i,
	                        (LPARAM)&(szBuf[i]) );
						for(j=i;i<BUF_SIZE && szBuf[i] && szBuf[i]!='\t';i++);
						if ( szBuf[i] ) { szBuf[i] = '\0'; };
						lAdmins.Append( (DblList::type)PI_NEW(
	                        PIString( &(szBuf[j]) ) ) );
						szBuf[i++] = '\t';
	
						::SendMessage( hWndPassword, WM_GETTEXT, BUF_SIZE-i,
	                        (LPARAM)&(szBuf[i]) );
						for(j=i; i<BUF_SIZE && szBuf[i] && szBuf[i]!='\t'; i++);
						if ( szBuf[i] ) { szBuf[i] = '\0'; };
						lPasswords.Append( (DblList::type)PI_NEW(
                            PIString( &(szBuf[j]) ) ) );
	
						/*
						** Delete the item from the list and internal lists
						*/
						::SendMessage( hWndList, LB_DELETESTRING, iCurSel, 0 );
                        PI_DELETE( (PIString *)lHosts.Detach( iCurSel) );
						PI_DELETE( (PIString *)lServerRoots.Detach( iCurSel) );
						PI_DELETE( (PIString *)lAdmins.Detach( iCurSel));
						PI_DELETE( (PIString *)lPasswords.Detach(iCurSel));
						
						/*
						** Add new item to listbox
						*/
						::SendMessage(hWndList, LB_ADDSTRING, 0, (LPARAM)szBuf);
	
						/*
						** Clear edit fields, focus to list
						*/
                        ::SendMessage( hWndHost, WM_SETTEXT, 0, (LPARAM)"");
						::SendMessage( hWndServerRoot, WM_SETTEXT,0,(LPARAM)"");
						::SendMessage( hWndAdmin, WM_SETTEXT, 0,(LPARAM)"");
						::SendMessage( hWndPassword,WM_SETTEXT, 0,(LPARAM)"");
						::SendMessage( hWndConfirm,WM_SETTEXT, 0,(LPARAM)"");
						Internal_reloadArrays();
						int iIndex = ::SendMessage(hWndList, LB_FINDSTRINGEXACT,
                            0, (LPARAM)szBuf );
						if ( iIndex!=LB_ERR ) ::SendMessage( hWndList,
	                        LB_SETTOPINDEX, iIndex, 0 );
						::SetFocus( hWndList );
						PropSheet_Changed( ::GetParent( hDlg ), hDlg );
						};
						break;
	
					case IDC_DELETE:
						{
						int iCurSel = ::SendMessage( hWndList, LB_GETCURSEL,
                            0, 0 );
						SendMessage( hWndList, LB_DELETESTRING,	iCurSel, 0 );
                        PI_DELETE( (PIString *)lHosts.Detach( iCurSel) );
						PI_DELETE( (PIString *)lServerRoots.Detach( iCurSel) );
						PI_DELETE( (PIString *)lAdmins.Detach(iCurSel));
						PI_DELETE( (PIString *)lPasswords.Detach(iCurSel));
				
						/*
						** Clear edit fields, focus to list
						*/
                        ::SendMessage( hWndHost, WM_SETTEXT, 0, (LPARAM)"" );
						::SendMessage( hWndServerRoot, WM_SETTEXT,0,(LPARAM)"");
						::SendMessage( hWndAdmin, WM_SETTEXT, 0, (LPARAM)"" );
						::SendMessage( hWndPassword, WM_SETTEXT, 0,(LPARAM)"" );
						::SendMessage( hWndConfirm, WM_SETTEXT, 0,(LPARAM)"" );
						::SetFocus( hWndList );
					    PropSheet_Changed( ::GetParent( hDlg ), hDlg );
					    };
					    break;

				case IDC_HOST:
					*szBuf = '\0';
					SendMessage( hWndHost,
                        WM_GETTEXT, BUF_SIZE, (LPARAM)szBuf );
					Internal_updateButtons( wParam, lParam, hWndServerRoot, 
						hWndHost, szBuf );
                    break;

				case IDC_SERVERROOT:
					*szBuf = '\0';
					SendMessage( hWndHost,
                        WM_GETTEXT, BUF_SIZE, (LPARAM)szBuf );
					Internal_updateButtons( wParam, lParam, hWndHost, 
						hWndServerRoot, szBuf );
                    break;

				case IDC_PASSWORD:
					*szBuf = '\0';
					SendMessage( hWndHost,
                        WM_GETTEXT, BUF_SIZE, (LPARAM)szBuf );
					Internal_updateButtons( wParam, lParam, hWndHost, 
						hWndServerRoot, szBuf );
					break;

				case IDC_ADMINISTRATOR:
					*szBuf = '\0';
					SendMessage( hWndHost,
                        WM_GETTEXT, BUF_SIZE, (LPARAM)szBuf );
					Internal_updateButtons( wParam, lParam, hWndHost, 
						hWndServerRoot, szBuf );
					break;

				case IDC_CONFIRM:
					*szBuf = '\0';
					SendMessage( hWndHost,
                        WM_GETTEXT, BUF_SIZE, (LPARAM)szBuf );
					Internal_updateButtons( wParam, lParam, hWndHost, 
						hWndServerRoot, szBuf );
				    break;

				case IDC_LIST:
                    if ( GET_WM_COMMAND_CMD( wParam, lParam )==LBN_SELCHANGE )
						{
						int iSel = ::SendMessage( hWndList, LB_GETCURSEL, 0,0);
						if ( iSel!=LB_ERR )
							{
							assert( iSel>=0 && iSel<lHosts.Size() );
						    ::SendMessage( hWndHost, WM_SETTEXT, 0,
							    (LPARAM)(const char *)*((PIString *)
                                    lHosts[iSel]) );
							::SendMessage( hWndServerRoot, WM_SETTEXT, 0,
								(LPARAM)(const char *)*((PIString *)
                                    lServerRoots[iSel]) );
							::SendMessage( hWndAdmin, WM_SETTEXT, 0,
								(LPARAM)(const char *)*((PIString *)
                                    lAdmins[iSel]) );
							::SendMessage( hWndPassword, WM_SETTEXT, 0,
								(LPARAM)(const char *)*((PIString *)
                                    lPasswords[iSel]) );
							::SendMessage( hWndConfirm, WM_SETTEXT, 0,
								(LPARAM)(const char *)*((PIString *)
                                    lPasswords[iSel]) );
                            ::SetFocus( hWndList );
							};
						};
					Internal_updateButtons( wParam, lParam, hWndHost, 
						hWndServerRoot, szBuf );
					break;

				default:;		
				}
			break;

		    case WM_NOTIFY:
			    {
			    switch( ((NMHDR *)lParam)->code )
				    {
				    case PSN_QUERYCANCEL:
					    return Close( hDlg );

				    case PSN_KILLACTIVE:
					    {
					    int i;
					    int iCount = SendMessage( hWndList, LB_GETCOUNT, 0, 0 );

					    /*
					    ** Save all date
					    */
					    /* --- remove old ones first --- */
					    M_DeleteValue( pConfig, sSection, VARIABLE );
					    for( i=0; i<iCount; i++ )
						    {
						    *szBuf = '\0';
						    szBuf[BUF_SIZE] = '\0';
						    ::SendMessage( hWndList, LB_GETTEXT, i,
                                (LPARAM)szBuf );
						    M_AddValue( pConfig, sSection, VARIABLE, szBuf );
						    };
					    };
					::SetWindowLong( hDlg, DWL_MSGRESULT, FALSE );
					break;

					default:;
					};
				};
				break;

                default:
                    return (FALSE);
            };

        return (TRUE);
        };
};

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
static BOOL CALLBACK fnDialogProc(HWND hDlg, UINT uMsg, WPARAM wParam,
    LPARAM lParam)
{
    if ( uMsg==WM_INITDIALOG )
        {
        ::SetWindowLong( hDlg, DWL_USER, ((LPPROPSHEETPAGE)lParam)->lParam );
        };

    VServersPage *pPage = (VServersPage *)::GetWindowLong( hDlg, DWL_USER );
    return pPage ? pPage->DialogProc( hDlg, uMsg, wParam, lParam ) : FALSE;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int VServersPage_constructor( PIObject *pObj,
	int iArgc, const char *ppArgv[] )
{
	return IFaceBase_constructor( pObj, PI_NEW( VServersPage( pObj,
		iArgc, ppArgv ) ) );
}

#if 0
/*___+++CNF_BEGIN+++___*/
	<Class>
		Name VServersPageClass
		Type LogicExtension
		Library IntrFace
		OnClassLoad IFaceBase_onClassLoad
		Constructor VServersPage_constructor
		CopyConstructor IFaceBase_copyConstructor
		Destructor IFaceBase_destructor
		Execute IFaceBase_execute
	</Class>

	<Object>
		Name IPVServersPage
		Class VServersPageClass
        Flag "IP"
	</Object>

	<Object>
		Name NonIPVServersPage
		Class VServersPageClass
        Flag "NonIP"
	</Object>

/*___+++CNF_END+++___*/
#endif

