/*____________________________________________________________________________*\
 *

 Copyright (c) 1997 John Roy. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
    mentioning features must contain the following
    acknowledgment:
    "This product includes software developed by John Roy
    (http://www.johnroy.com/pi3/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: _PIIO.h$
 * $Date: Sun Aug 10 06:42:20 1997$
 *
 Description:
	Implementation of Socket layer API functions.
\*____________________________________________________________________________*/
//$SourceTop:$

#ifndef _PIIO_H_
#define _PIIO_H_

#include <assert.h>
#include "_PIClass.h"

/*____________________________________________________________________________*\
 *
 Class:
 Description:
\*____________________________________________________________________________*/
class _PIIOClass : public _PIClass
{
public:
	enum Function {
		IN=0,
		OUT,

		/* this must always be last */
		LAST_FUNCTION
		};

private:
	_PIClass::PfnFunction aFunctionFrame[ LAST_FUNCTION ];

public:
	_PIIOClass(
        PIDLL *pDll,
		const char *pClassName,
		const char *pClassDescription,
		PfnFunction *pFunctions,
		_PIDB *pDB )
	: _PIClass(
            pDll,
			pClassName,
			pClassDescription,
			pFunctions,
			pDB )
		{
		assert( pFunctions );
		memcpy(
			aFunctionFrame,
			&(pFunctions[_PIClass::LAST_FUNCTION]),
			sizeof( aFunctionFrame ) );
		};


	~_PIIOClass()
		{
		};

	virtual const char *GetClassName() const
		{
		return PIDBVALUE_CONF_IO; 
		};

	virtual Type GetType() const
		{ return IO; };

	/* Implement function calls */
	inline int out( _PIObject *pObj, void *pData, 
		int iLen, int *piLen, int iArgc, const char *ppArgv[] ) const
		{
		return ( 
			( int (*)(_PIObject *, void *, int , int *, int, const char *[] ) )
			( aFunctionFrame[OUT] ))( pObj, pData, iLen, piLen, iArgc, ppArgv );
		};

	inline int in( _PIObject *pObj, void *pData, 
		int iLen, int *piLen, int iArgc, const char *ppArgv[] ) const
		{
		return (
			( int (*)(_PIObject *, void *, int , int *, int, const char *[] ) )
			( aFunctionFrame[IN] )) ( pObj, pData, iLen, piLen, iArgc, ppArgv );
		};
};

#endif	/* _PIIO_H_ */

