/*____________________________________________________________________________*\
 *

 Copyright (c) 1997 John Roy. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
    mentioning features must contain the following
    acknowledgment:
    "This product includes software developed by John Roy
    (http://www.johnroy.com/pi3/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: _PIObj.cpp$
 * $Date: Sun Aug 10 06:42:21 1997$
 *
 Description:
\*____________________________________________________________________________*/
//$SourceTop:$

#include <ctype.h>

#include "StrToken.h"
#include "_PIObj.h"
#include "AutoDel.h"
#include "PIStrStr.h"
#include "_PIDBUtl.h"
#include "DeQuote.h"

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
_PIObject *_PIObject::Load( PIDB *pDB, PIConfigDB *pConfig,
	const char *pObjectName, int iArgc, const char *ppArgv[] )
{
	assert( pDB && pObjectName );

	PIConfigDB *pConf = PIConfigDB::LookupObjectWithName( pDB, pConfig, 
		pObjectName );

	if ( !pConf )
		{   
		PIOStrStream os;
		os << "Unable to load object '" << pObjectName << "', configuration \
definition could not be found." << ends;
		_PIDBUtil::AddLogMessage( pDB, pConfig, PILOG_ERROR, os.str() );
		return 0;
		};

	/* --- get the name of the class --- */
	const char *pClassName = PIConfigDB::LookupValue( pConf, 
		PIDBKEY_CONF_CLASS, 0, 0 );

	if ( !pClassName )
		{
		PIOStrStream os;
		os << "Unable to load object '" << pObjectName << "', configuration \
definition is missing a definition for the object's 'Class'." << ends;
		_PIDBUtil::AddLogMessage( pDB, pConfig, PILOG_ERROR, os.str() );
		return 0;
		};

	const _PIClass *pClass = _PIClass::LookupOrLoad( pDB, pConfig, pClassName );
	if ( !pClass )
		{
		return 0;
		};

	/* --- correct the name the of object, wrt case --- */
	pObjectName = PIConfigDB::LookupValue( pConf, PIDBKEY_CONF_NAME, 0, 0 );

	/* --- 
	Check for object attributes
	NOTE: This is antiquated 
	--- */
	const char *pTmp = PIConfigDB::LookupValue( pConf, PIDBKEY_CONF_ATTRIBUTE,
		0, 0 );
	bool bHasTree = pTmp && !pi_stricmp( pTmp, PIDBVALUE_CONF_HASTREE );

#if 0
	/* ---
	Check for nested objects
	NOTE: Removed because its antiquated 
	--- */
	_PIDBIterator *pIter = 0;
	AutoDelete<_PIDBIterator> tA( pIter = pConf->GetIterator( 
		PIDBTYPE_CONF_STRING, PIDBKEY_CONF_OBJECT, 0 ) );
	for(; pIter && pIter->AtValidElement(); pIter->Next() )
		{
		PIConfigDBString *pString = (PIConfigDBString *)pIter->Current( 0 );
		if ( !pString ) { continue; };
		const char *pValue = pString->GetString();
		};
#endif

	if ( bHasTree )
		{
		pDB = PI_NEW( PIDBTree( pDB, pObjectName ) );
		};

	_PIObject *pObj = PI_NEW( _PIObject( pObjectName, pClass, pDB, pConf ) );
	if ( pClass->Constructor( pObj, iArgc, ppArgv ) )
		{
		PIOStrStream os;
		os << "Failed to initialize object '" << pObj->GetName()
			<< "' of class '" << pClass->GetName() << "'." << ends;
		_PIDBUtil::AddLogMessage( pDB, pConfig, PILOG_ERROR, os.str() );
		Delete( pObj, 0, 0 );
		pObj = 0;
		if ( bHasTree )
			{
			pDB->Detach();
			PI_DELETE( pDB );
			};
		};
	return pObj;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
_PIObject *_PIObject::LoadFromLine( PIDB *pDB, PIConfigDB *pConfigDB,
	const char *pObjectLine )
{
	if ( !pObjectLine )
		{
		assert( 0 );
		return 0;
		};

	if ( !pDB || !pConfigDB )
		{
		PILog_addMessage( pDB, pConfigDB, PILOG_ERROR, "\
Object has no DB or Configuration DB" );
		return 0;
		};

	StringTokenizer tTokens( pObjectLine, " ", 0 );
	int iNumTokens = tTokens.NumTokens();
	if ( iNumTokens<1 )
		{
		PIOStrStream os;
		os << "Object line is empty" << ends;
		PILog_addMessage( pDB, pConfigDB, PILOG_ERROR, os.str() );
		return 0;
		};
	const char *pObjectName = tTokens.GetToken( 0 );
	int iArgc = iNumTokens-1;
	char **ppArgv = 0;
	if ( iArgc>0 )
		{
		ppArgv = PI_NEW( char *[iArgc] ); 
		for( int i=0; i<iArgc; i++ )
			{
			ppArgv[i] = (char *)(const char *)tTokens.GetToken( i+1 );
			};
		};

	PIObject *pNewObject = Load( pDB, 0, pObjectName, iArgc,
		(const char **)ppArgv ); 
	PI_DELETE( [] ppArgv );
	if ( !pNewObject )
		{
		/* --- Message generated by Load() --- */
		return 0;
		};

	return pNewObject;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
_PIObject *_PIObject::LoadEx( const _PIObject *pObject,
	const char *pObjectKey )
{
	if ( !pObject || !pObjectKey )
		{
		assert( 0 );
		return 0;
		};

	PIDB *pDB = PIObject_getDB( pObject );
	PIConfigDB *pConfigDB = PIObject_getConfigurationDB( pObject );
	if ( !pDB || !pConfigDB )
		{
		PILog_addMessage( pDB, pConfigDB, PILOG_ERROR, "\
Object has no DB or Configuration DB" );
		return 0;
		};
	const char *pObjectLine = PIConfig_lookupValue( pConfigDB,
		pObjectKey, 0, 0 );
	if ( !pObjectLine )
		{
		PIOStrStream os;
		os << "No definition for '" << pObjectKey << "'" << ends; 
		PILog_addMessage( pDB, pConfigDB, PILOG_ERROR, os.str() );
		return 0;
		};

	return _PIObject::LoadFromLine( pDB, pConfigDB, pObjectLine );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
#define CONFIG_ERR(this, msg)   \
	{ PILog_addMessage( PIObject_getDB(this), \
	PIObject_getConfigurationDB(this), PILOG_ERROR, (msg) ); }
int _PIObject::ReadParameters(
	PIObject *pObject,
	int iArgc,
	const char *ppArgv[],
	int (* fnParameter)( void *, const char *, const char *, const char *),
	void *pParam )
{
	assert( pObject && fnParameter );
	if ( !( pObject && fnParameter ) )
		{
		return 0;
		};

	/* --- loop over parameters passed on line --- */
	if ( ppArgv )
		{
		for( int i=0; i<iArgc; i++ )
			{
			const char *pLine = ppArgv[i];
			if ( !pLine )
				{ continue; };
	
			/* --- remove leading whitespace --- */
			for( ; (isspace(*pLine)); pLine++ );
			
			int j=0;

			/* --- scan to first '=' --- */
			for( ; pLine[j] && pLine[j]!='='; j++ );  

			/* --- grap variable name --- */
			PIString sVariable( pLine, j );

			/* --- skip the '=' --- */
			if ( pLine[j] )
				{ j++; };
			
			DeQuote tUnQuoted( &(pLine[j]) );

			if ( !(fnParameter)( pParam, sVariable, tUnQuoted, "" ) )
				{
				return 0;	
				};
			};
		};

	PIDBIterator *pIter = PIDB_getIterator(
		(PIDB *)PIObject_getConfigurationDB( pObject ),
		PIDBTYPE_USER, 0, 0 );
	
	if ( !pIter ) 
		{
		CONFIG_ERR( pObject, "Could not read values for object" );
		PIDBIterator_delete( pIter );
		return 0;
		};

	for(; PIDBIterator_atValidElement( pIter ); PIDBIterator_next( pIter ))
		{
		const char *pVariable = 0;
		PIConfigString *pString = (PIConfigString *)PIUserType_getValue(
			(PIUserType *)PIDBIterator_current( pIter, &pVariable ) );
		const char *pValue = PIConfigString_getString( pString );

		/* --- generate location string --- */
		const char *pFile = pString->GetFileName();
		PIOStrStream osLocation;
		if ( pFile )
			{
			long lLine = pString->GetY();
			
			/* --- if line position is 0, this is the previous line --- */
			if ( !pString->GetX() && lLine>0 )
				{ lLine--; };

			osLocation << "[" << pFile << ", line " << lLine << "] " << ends;
			};

		/* --- no variable --- */
		if ( !pVariable || !*pVariable )
			{
			osLocation << "ReadParameters: Internal error reading \
variable." << ends;
			CONFIG_ERR( pObject, osLocation.str() );
			PIDBIterator_delete( pIter );
			return 0;
			};

		/* --- no value --- */
		if ( !pValue || !*pValue )
			{
			osLocation << "ReadParameters: no value for \
variable '" << pVariable << "'" << ends;
			CONFIG_ERR( pObject, osLocation.str() );
			PIDBIterator_delete( pIter );
			return 0;
			};

		/* --- ignore 'Name' or 'Class' parameters --- */
		if (	!PIUtil_stricmp( PIDBKEY_CONF_NAME, pVariable ) ||
				!PIUtil_stricmp( PIDBKEY_CONF_CLASS, pVariable ) )
			{ continue; };

			
		DeQuote tUnQuoted( pValue );

		if ( !(fnParameter)( pParam, pVariable, tUnQuoted, osLocation.str() ) ) 
			{
			return 0;	
			};
		};
	PIDBIterator_delete( pIter );

	return 1;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
_PIObject *PIObject::New( const _PIClass *pTheClass, PIDB *pDB,
	PIConfigDB *pConf, const char *pObjectName, int iArgc,
	const char *ppArgv[] )
{
	assert( pTheClass && pDB && pObjectName );
	if ( !pTheClass || !pDB || !pObjectName )
		{ return 0; };

	_PIObject *pObj = PI_NEW( _PIObject(pObjectName, pTheClass, pDB, pConf ) );
	if ( pTheClass->Constructor( pObj, iArgc, ppArgv ) )
		{
		Delete( pObj, 0, 0 );
		pObj = 0;
		};
	return pObj;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int _PIObject::Delete( _PIObject *pObj, int iArgc, const char *ppArgv[] )
{
	assert( pObj && pObj->GetClass() );
	int iRet = pObj->GetClass()->Destructor( pObj, iArgc, ppArgv );
	PI_DELETE( pObj );
	return iRet;
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../piapi for full description
\*____________________________________________________________________________*/
_PIObject *_PIObject::Copy( const _PIObject *pObject, int iArgc,
	const char *ppArgv[] )
{
	assert( pObject );
    _PIObject *pNewObject = PI_NEW( _PIObject( *pObject ) );
    if ( pNewObject->GetClass()->CopyConstructor( pNewObject, iArgc, ppArgv ) )
        {
        Delete( pNewObject, 0, 0 );
        pNewObject = 0;
        };
    return pNewObject;
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../piapi for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI PIObject *PIObject_load( PIDB *pDB, PIConfigDB *pConfig, 
	const char *pName, int iArgc, const char *ppArgv[] )
{
	if ( !pDB || !pName )
		{ return 0; };
	return _PIObject::Load( pDB, pConfig, pName, iArgc, ppArgv );
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../piapi for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI PIObject *PIObject_loadFromLine( PIDB *pDB, PIConfigDB *pConfig, 
	const char *pObjectLine )
{
	if ( !pDB || !pObjectLine || !*pObjectLine )
		{ return 0; };
	return _PIObject::LoadFromLine( pDB, pConfig, pObjectLine );
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../piapi for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI PIObject *PIObject_loadEx( const PIObject *pParent,
	const char *pObjectKey )
{
	if ( !pParent || !pObjectKey )
		{ return 0; };
	return _PIObject::LoadEx( pParent, pObjectKey );
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../piapi for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI int PIObject_readParameters(
	PIObject *pObject,
	int iArgc,
	const char *ppArgv[],
	int (* fnParameter)( void *, const char *, const char *, const char *),
	void *pParam )
{
	return _PIObject::ReadParameters( pObject, iArgc, ppArgv, fnParameter,
		pParam );
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../piapi for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI PIObject *PIObject_new( const PIClass *pClass, PIDB *pDB,
	PIConfigDB *pConfig, const char *pName, int iArgc, const char *ppArgv[] )
{
	if ( !pClass || !pDB || !pName )
		{ return 0; };
	return _PIObject::New( pClass, pDB, pConfig, pName, iArgc, ppArgv );
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../piapi for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI _PIObject *PIObject_copy( const _PIObject *pObject,
	int iArgc, const char *ppArgv[] )
{
	if ( !pObject )
		{ return 0; };
	return _PIObject::Copy( pObject, iArgc, ppArgv );
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../piapi for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI const char *PIObject_getName( const PIObject *pObj )
{
	if ( !pObj )
		{ return 0; };
	return pObj->GetName();
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../piapi for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI PIDB *PIObject_getDB( const PIObject *pObj )
{
	if ( !pObj )
		{ return 0; };
	return pObj->GetDB();
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../piapi for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI int PIObject_setDB( PIObject *pObj, PIDB *pDB )
{
	if ( !pObj )
		{ return PIAPI_EINVAL; };
	pObj->SetDB( pDB );
	return PIAPI_COMPLETED;
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../piapi for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI PIConfigDB *PIObject_getConfigurationDB( const PIObject *pObj )
{
	if ( !pObj )
		{ return 0; };
	return pObj->GetConfigurationDB();
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../piapi for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI int PIObject_setConfigurationDB( PIObject *pObj,
	PIConfigDB *pConfig )
{
	if ( !pObj )
		{ return PIAPI_EINVAL; };
	pObj->SetConfigurationDB( pConfig );
	return PIAPI_COMPLETED;
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../piapi for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI const PIClass *PIObject_getClass( const PIObject *pObj )
{
	if ( !pObj )
		{ return 0; };
	return pObj->GetClass();
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../piapi for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI void *PIObject_getUserData( const PIObject *pObj )
{
	if ( !pObj )
		{ return 0; };
	return pObj->GetUserData();
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../piapi for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI int PIObject_setUserData( PIObject *pObj, void *pData )
{
	if ( !pObj )
		{ return PIAPI_EINVAL; };
	pObj->SetUserData( pData );
	return PIAPI_COMPLETED;
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../piapi for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI int PIObject_delete( PIObject *pObj, int iArgc,
	const char *ppArgv[] )
{
	if ( !pObj )
		{ return PIAPI_COMPLETED; };
	return _PIObject::Delete( pObj, iArgc, ppArgv );
}

