/*____________________________________________________________________________*\
 *

 Copyright (c) 1997 John Roy. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
    mentioning features must contain the following
    acknowledgment:
    "This product includes software developed by John Roy
    (http://www.johnroy.com/pi3/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: ConnPool.cpp$
 * $Date: Sun Aug 10 06:36:32 1997$
 *
 Description:
\*____________________________________________________________________________*/
//$SourceTop:$

#include <assert.h>

#include "PICompat.h"
#include "ConnPool.h"

/*____________________________________________________________________________*\
 *
 Class:
 Description:
\*____________________________________________________________________________*/
class _ConnectionPool
{
private:
    PIObject *pIOObject;

public:
    _ConnectionPool( PIObject *pTheIOObject )
    :   pIOObject( pTheIOObject )
        {
        };

    ~_ConnectionPool()
        {
        PIObject_delete( pIOObject, 0, 0 );
        };

    PIIOBuffer *GrapConnection( const char *pHost, int iPort )
        {
        /* --- establish a connection to the remote server --- */
        const char *ppConnectParameters[2];
        ppConnectParameters[0] = pHost;
        ppConnectParameters[1] = (const char *)iPort;
        PIObject *pConn = PIObject_copy( pIOObject, 2, ppConnectParameters );
        PIIOBuffer *pNewBuffer = PIIOBuffer_new( pConn );
        PIIOBuffer_setUserData( pNewBuffer, pConn );
        return pNewBuffer;
        };

    bool ReleaseConnection( PIIOBuffer *pIOBuffer )
        {
        PIObject *pIOObject = (PIObject *)PIIOBuffer_getUserData( pIOBuffer );
        PIIOBuffer_delete( pIOBuffer );
        PIObject_delete( pIOObject, 0, 0 );
        return true;
        };
};

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI ConnectionPool *ConnectionPool_new( PIObject *pIOObject, 
    int iMaxSize )
{
    (void)iMaxSize;
    return PI_NEW( ConnectionPool( pIOObject ) );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int ConnectionPool_delete( ConnectionPool *pConnectionPool )
{
    if ( !pConnectionPool )
        {
        return PIAPI_EINVAL;
        };

    PI_DELETE( pConnectionPool );
    return PIAPI_COMPLETED;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI PIIOBuffer *ConnectionPool_grapConnection(
    ConnectionPool *pConnectionPool, const char *pHost, int iPort )
{
    if ( !pConnectionPool )
        {
        return 0;
        };

    return pConnectionPool->GrapConnection( pHost, iPort ); 
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int ConnectionPool_releaseConnection(
    ConnectionPool *pConnectionPool, PIIOBuffer *pConnection, bool bMustClose )
{
    if ( !pConnectionPool )
        {
        return 0;
        };

    (void)bMustClose;
    bool bRet = pConnectionPool->ReleaseConnection( pConnection );
    return bRet ? PIAPI_COMPLETED : PIAPI_ERROR;
}

