/*____________________________________________________________________________*\
 *

 Copyright (c) 1997 John Roy. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
    mentioning features must contain the following
    acknowledgment:
    "This product includes software developed by John Roy
    (http://www.johnroy.com/pi3/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: PIIOBuf.cpp$
 * $Date: Sun Aug 10 06:36:32 1997$
 *
 Description:
	TODO:
		- really need some output buffering here
\*____________________________________________________________________________*/
//$SourceTop:$

#include <assert.h>
#include <string.h>
#include <iostream.h>

#include "PICompat.h"
#include "PIIOBuf.h"
#include "HTTPDefs.h"

/*
#define D { cerr << __FILE__ << ": " << __LINE__ << endl; };
*/
#define D

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
inline int MyStrLen( const char *pS )
	{ return (int)strlen( pS ); };

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
	Flush the output buffer. Returns >0 on success, 0 on failure.
\*____________________________________________________________________________*/
int PIIOBuffer::FlushOutput()
{
	if ( iOutBufferSize==0 )
		{ return 1; };

	assert( pOutBuffer );
	int iWritten = 0;
	if ( iOutIOError )
		{ return 0; };
	if ( PIIO_out( pIOObject, (void *)pOutBuffer, iOutBufferSize, &iWritten,
		0, 0 ) )
		{ iOutIOError = 1; return 0; };
	assert( iWritten<=iOutBufferSize );
	iBytesSent += iWritten;
	if ( iWritten<iOutBufferSize )
		{
		memcpy( pOutBuffer, &( pOutBuffer[iWritten] ),
			iOutBufferSize-iWritten );
		iOutBufferSize -= iWritten;
		int iRet = FlushOutput();
		if ( !iRet )
			{ return 0; };
		iWritten += iRet;
		};
	assert( iWritten<=iOutBufferSize );
	iOutBufferSize = 0;
	return iWritten;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
	Append output data to the internal buffer, or flush straight out depending
	on flags. Returns bytes written on success, 0 on failure.
\*____________________________________________________________________________*/
int PIIOBuffer::Internal_Write( const char *pS, int iLen, int iFlags )
{
	assert( pS );
	assert( pOutBuffer );	
	int iWritten = 0;

	/* --- buffering disabled and nothing currently in the buffer --- */
	if ( ( iFlags & PIIOBUF_NOBUFFER ) && ( iOutBufferSize==0) )
		{
		if ( iOutIOError )
			{ return 0; };
		if ( PIIO_out( pIOObject, (void *)pS, iLen,
			&iWritten, 0, 0 ) )
			{ iOutIOError = 1; return 0; };
		return iWritten;
		};

	/* --- data in the buffer --- */
	if ( iOutBufferSize<OUT_BUFFER_SIZE )
		{
		int iToWrite = OUT_BUFFER_SIZE-iOutBufferSize;

		if ( iToWrite>iLen )
			{ iToWrite=iLen; };

		memcpy( &( pOutBuffer[iOutBufferSize] ), pS, iToWrite );
		iOutBufferSize+=iToWrite;
		iLen -= iToWrite;
		if ( iFlags & PIIOBUF_NOBUFFER )
			{
			if ( !FlushOutput() )
				{
				return 0;
				};
			};
		if ( !iLen )
			{
			return iToWrite;
			};
		pS = &( pS[iToWrite] );
		iWritten += iToWrite;
		};

	if ( !FlushOutput() )
		{ return 0; };

	int iTmp = Internal_Write( pS, iLen, iFlags );
	if ( !iTmp )
		{ return 0; };
	return iWritten+iTmp;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:	private
 Description:
	Grows the data in the internal buffer pBuffer by reading additional
	data from the IO object. Returns length of data read or 0 if
	no more data is available. Returns -1 if the buffer is full or an IO error 
	occurred.
\*____________________________________________________________________________*/
int PIIOBuffer::GrowData()
{
	int iRead;
	if ( iBufferLen==IN_BUF_SIZE )
		{ return -1; };
		
	if ( PIIO_in(
		pIOObject,
		&(szInBuf[iBufferLen]),
		IN_BUF_SIZE-iBufferLen,
		&iRead,
		0, 0 ) )
		{ return -1; };
	iBufferLen += iRead;
	return iRead;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
	Poll for incoming data. If there is data in the read buffer, then
	return 1. If data was succesfully read from the input stream then
	return 1. Otherwise 0 is returned on a timeout condition. On error
	this function returns -1.
\*____________________________________________________________________________*/
int PIIOBuffer::PollBeforeRead()
{
	if ( iBufferLen )
		{ /* --- data in buffer --- */ return 1; };

	int iRet = GrowData();
	switch( iRet )
		{
		case 0:
		case -1:
			return iRet;

		default:
			return 1;
		};
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int PIIOBuffer::Write( const char *pS, int iLen, int iFlags )
{
	assert( pS );
	if ( iLen==-1 )
		{
		iLen = MyStrLen( pS );
		};
	int iWritten = 0;
	if ( iLen && !( iWritten = Internal_Write( pS, iLen, iFlags ) ) )
		{
		return -1;
		};
	return iWritten;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int PIIOBuffer::WriteLn( const char *pS, int iLen, int iFlags )
{
	assert( pS );
	if ( iLen==-1 )
		{ iLen = MyStrLen( pS ); };
	int iWritten = 0;
	if ( iLen && !( iWritten = Internal_Write( pS, iLen, iFlags ) ) )
		{ return -1; };
	int iTotal = iWritten;
	iWritten = Internal_Write( HTTP_CRLF, 2, iFlags );
	if ( !iWritten )
		{ return -1; };
	iTotal += iWritten;
	return iTotal;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
	Return the internal buffer, and put length into the integer at piRead.
	Return 0 if no data if available or an error occurs.
\*____________________________________________________________________________*/
const char *PIIOBuffer::Read( int *piRead )
{
	assert( piRead );
	if ( !piRead )
		{ return 0; };

	/* ---
	If there is data in the buffer, use it
	--- */
	if ( iBufferLen )
		{
		*piRead = iBufferLen;
		iBufferLen = 0;
		return szInBuf;
		};

	/* ---
	Otherwise read in some more
	--- */
	switch( GrowData() )
		{
		case 0:		/* --- no more data --- */
		case -1:	/* --- IO error --- */
			return 0;

		default:
			/* got more data */
			;
		};

	/* --- recurse to return the data block --- */
	return Read( piRead );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
	Write up to iMaxLen characters into the buffer pszBuffer from
	the input and return the number of characters written.
	Returns 0 on EOF or -1 on error.
\*____________________________________________________________________________*/
int PIIOBuffer::Read( char *pszBuffer, int iMaxLen )
{
	assert( pszBuffer && iMaxLen );
	if ( !pszBuffer || !iMaxLen )
		{ return -1; };

	/* ---
	If there is data in the buffer, use it
	--- */
	if ( iBufferLen )
		{
		if ( iBufferLen>iMaxLen )
			{	
			/* ---
			More characters are in the buffer than we want to read.
			--- */
			memcpy( pszBuffer, szInBuf, iMaxLen );
			iBufferLen -= iMaxLen;
			memmove( szInBuf, &(szInBuf[iMaxLen]), iBufferLen );
			return iMaxLen;
			}
		else
			{
			memcpy( pszBuffer, szInBuf, iBufferLen );
			int iTmp = iBufferLen;
			iBufferLen = 0;
			return iTmp;
			};
		};

	/* ---
	Otherwise read in some more
	--- */
	switch( GrowData() )
		{
		case 0:		/* --- no more data --- */
			return 0;
		case -1:	/* --- IO error --- */
			return -1;

		default:
			/* got more data */
			;
		};

	/* --- recurse to return the data block --- */
	return Read( pszBuffer, iMaxLen );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
	Returns the next line from the IO Object. If pszBuf is not
	NULL then the line is written into it, up to iMaxLen bytes.
	Returns the length of the line read.

	Will non-terminate line.
\*____________________________________________________________________________*/
int PIIOBuffer::GetLine( char *pszBuf, int iMaxLen )
{
	assert( pszBuf && iMaxLen );
	if ( !pszBuf || !iMaxLen )
		{ return 0; };
	enum { NORMAL, CR_SEEN, LF_SEEN, CRLF_SEEN } eState = NORMAL;
	int i;
	int iBlockStart = 0;		/* --- start of last block appended by
										GrowData() --- */
	int iLineLen = 0;			/* --- length of line --- */
	for(;;)
		{
		/* ---
		Scan this block for EOL 
		--- */
		for( i=iBlockStart; i<iBufferLen; i++)
			{
			switch( eState )
				{
				case NORMAL:
					if ( szInBuf[i]==HTTP_CR )
						{ eState = CR_SEEN; }
					else if ( szInBuf[i]==HTTP_LF )
						{
						eState = LF_SEEN;
						i++; goto done;
						};
					break;

				case CR_SEEN:
					if ( szInBuf[i]==HTTP_LF )
						{
						eState = CRLF_SEEN;
						i++; goto done;
						};
					i++; goto done;
					break;

				default:
					assert( 0 );
				};
			};

		/* i is now pointing at the char after the last valid char */

		/* ---
		This block has been exhausted without reading till EOL, try to
		read in more data
		--- */
		switch( GrowData() )
			{
			case 0:
				/* no more data */
				goto done;
				break;
			case -1:
				/* buffer full or IO error */
				goto done;
				break;
			default:
				/* got more data */
				;
			};
		};
done:
	if ( !i )
		{ goto do_return; };
	iLineLen = i;
	if ( pszBuf )
		{
		if ( iMaxLen < i )
			{ i = iMaxLen; };
		assert( i>=1 );
		if ( eState == CRLF_SEEN )
			{
			if ( i>=2 )
				{	
				i -= 2;
				strncpy( pszBuf, szInBuf, i );
				}
			else
				{ *szInBuf = '\0'; i=0; };
			}
		else if ( eState == LF_SEEN )
			{
			if ( i>=1 )
				{
				i -= 1;
				strncpy( pszBuf, szInBuf, i );
				}
			else
				{ *szInBuf = '\0'; i=0; };
			}
		else
			{
			strncpy( pszBuf, szInBuf, i );
			};
		assert( i<=iMaxLen );
		}

	/* ---
	Shrink the buffer for the next GetLine()
	--- */
	if ( iLineLen<iBufferLen )
		{
		memmove( szInBuf, &( szInBuf[iLineLen] ), iBufferLen-iLineLen );
		}
	else
		{		
		assert( iLineLen<=iBufferLen );
		};
	iBufferLen -= iLineLen;

do_return:
	pszBuf[i] = '\0';

	return i;
}

	/* -------------- +++++++++++++++++++++++++++++++ ------------------ *\
	
						C Functions

	\* -------------- +++++++++++++++++++++++++++++++ ------------------ */
/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PIIOBuffer *PIIOBuffer_new( PIObject *pIOObject )
{
	if ( !pIOObject ) 
		{ return 0; };
	return PI_NEW( PIIOBuffer( pIOObject ) );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int PIIOBuffer_delete( PIIOBuffer *pPIIOBuffer )
{
	if ( !pPIIOBuffer )
		{ return PIAPI_ERROR; };
	PI_DELETE( pPIIOBuffer );
	return PIAPI_COMPLETED;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int PIIOBuffer_getLine( PIIOBuffer *pIOBuf, char *pszBuf, int iBufLen )
{
	if ( !pIOBuf )
		{ return -1; };
	return pIOBuf->GetLine( pszBuf, iBufLen );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int PIIOBuffer_pollBeforeRead( PIIOBuffer *pIOBuf )
{
	if ( !pIOBuf )
		{ return -1; };
	return pIOBuf->PollBeforeRead();
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
const char *PIIOBuffer_read( PIIOBuffer *pIOBuf, int *piRead )
{
	if ( !pIOBuf )
		{ return 0; };
	return pIOBuf->Read( piRead );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int PIIOBuffer_readToBuffer( PIIOBuffer *pIOBuf, char *pszBuf, int iMaxLen )
{
	if ( !pIOBuf )
		{ return -1; };
	return pIOBuf->Read( pszBuf, iMaxLen );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int PIIOBuffer_write( PIIOBuffer *pIOBuf, const char *pszBuf, int iLen,
	int iFlags )
{
	if ( !pIOBuf )
		{ return -1; };
	return pIOBuf->Write( pszBuf, iLen, iFlags );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int PIIOBuffer_writeLn( PIIOBuffer *pIOBuf, const char *pszBuf, int iLen,
	int iFlags )
{
	if ( !pIOBuf )
		{ return -1; };
	return pIOBuf->WriteLn( pszBuf, iLen, iFlags );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int PIIOBuffer_flush( PIIOBuffer *pIOBuf )
{
	if ( !pIOBuf )
		{ return 0; };
	return pIOBuf->Flush();
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int PIIOBuffer_getOutputBuffer( PIIOBuffer *pIOBuf, char **ppBuffer )
{
	if ( !pIOBuf )
		{ return -1; };
	return pIOBuf->GetOutputBuffer( ppBuffer );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int PIIOBuffer_advanceBufferPointer( PIIOBuffer *pIOBuf, int iCount )
{
	if ( !pIOBuf )
		{ return -1; };
	return pIOBuf->AdvanceBufferPointer( iCount );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int PIIOBuffer_getBytesSent( PIIOBuffer *pIOBuf )
{
	if ( !pIOBuf )
		{ return -1; };
	return pIOBuf->GetBytesSent();
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
void PIIOBuffer_resetBytesSent( PIIOBuffer *pIOBuf )
{
	if ( !pIOBuf )
		{ return; };
	pIOBuf->ResetBytesSent();
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
void *PIIOBuffer_getUserData( PIIOBuffer *pIOBuf )
{
	if ( !pIOBuf )
		{ return 0; };
    return pIOBuf->GetUserData();
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
void PIIOBuffer_setUserData( PIIOBuffer *pIOBuf, void *pUserData )
{
	if ( !pIOBuf )
		{ return; };
    pIOBuf->SetUserData( pUserData );
}

