/*____________________________________________________________________________*\
 *

 Copyright (c) 1997 John Roy. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
    mentioning features must contain the following
    acknowledgment:
    "This product includes software developed by John Roy
    (http://www.johnroy.com/pi3/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: _PIFInfo.cpp$
 * $Date: Sun Aug 10 06:29:40 1997$
 *
 Description:
\*____________________________________________________________________________*/
//$SourceTop:$

#include "PiAPI.h"
#include "Platform.h"
#include "PlatDefs.h"

/*____________________________________________________________________________*\
*
 Description:
    See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI PIFInfo *PIFInfo_new( const char *pPath )
{
	if ( !pPath )
		{ PIERROR( PIAPI_EINVAL ); return 0; };
	PIString sPath( pPath );
	return (FileSystemEntity *)
		Platform::GetFile( (void *)&(sPath) ); //MSVC4.2 hack
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI int PIFInfo_delete( PIFInfo *pFileInfo )
{
	if ( !pFileInfo )
		{ PIERROR( PIAPI_EINVAL ); return PIAPI_EINVAL; };
	PI_DELETE( pFileInfo );
	return PIAPI_COMPLETED;
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI const char *PIFInfo_getName( PIFInfo *pFileInfo )
{
	if ( !pFileInfo )
		{ PIERROR( PIAPI_EINVAL ); return 0; };
	return pFileInfo->GetName();
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI const char *PIFInfo_getAlternateName( PIFInfo *pFileInfo )
{
	if ( !pFileInfo )
		{ PIERROR( PIAPI_EINVAL ); return 0; };
	return pFileInfo->GetAlternateName();
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI const char *PIFInfo_getPath( PIFInfo *pFileInfo )
{
	if ( !pFileInfo )
		{ PIERROR( PIAPI_EINVAL ); return 0; };
	return pFileInfo->GetPath();
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI const char *PIFInfo_getPathRoot( PIFInfo *pFileInfo )
{
	if ( !pFileInfo )
		{ PIERROR( PIAPI_EINVAL ); return 0; };
	return pFileInfo->GetPathRoot();
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI int PIFInfo_isRegular( PIFInfo *pFileInfo )
{
	if ( !pFileInfo )
		{ return PIAPI_FALSE; };
	return pFileInfo->IsRegularFile() ?  PIAPI_TRUE : PIAPI_FALSE;
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI int PIFInfo_exists( PIFInfo *pFileInfo )
{
	if ( !pFileInfo )
		{ return PIAPI_FALSE; };
	return pFileInfo->Exists() ?  PIAPI_TRUE : PIAPI_FALSE;
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI int PIFInfo_isDirectory( PIFInfo *pFileInfo )
{
	if ( !pFileInfo )
		{ return PIAPI_FALSE; };
	return pFileInfo->IsDirectory() ?  PIAPI_TRUE : PIAPI_FALSE;
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI int PIFInfo_isLink( PIFInfo *pFileInfo )
{
	if ( !pFileInfo )
		{ return PIAPI_FALSE; };
	return pFileInfo->IsLink() ?  PIAPI_TRUE : PIAPI_FALSE;
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI int PIFInfo_isReadable( PIFInfo *pFileInfo )
{
	if ( !pFileInfo )
		{ return PIAPI_FALSE; };
	return pFileInfo->IsReadable() ?  PIAPI_TRUE : PIAPI_FALSE;
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI int PIFInfo_isWritable( PIFInfo *pFileInfo )
{
	if ( !pFileInfo )
		{ return PIAPI_FALSE; };
	return pFileInfo->IsWritable() ?  PIAPI_TRUE : PIAPI_FALSE;
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI int PIFInfo_isExecutable( PIFInfo *pFileInfo )
{
	if ( !pFileInfo )
		{ return PIAPI_FALSE; };
	return pFileInfo->IsExecutable() ?  PIAPI_TRUE : PIAPI_FALSE;
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI PIPLATFORM_FSIZE PIFInfo_getSize( PIFInfo *pFileInfo )
{
	if ( !pFileInfo )
		{ PIERROR( PIAPI_EINVAL ); return 0; };
	PICLEAR;
	return (PIPLATFORM_FSIZE)pFileInfo->GetFileSize();
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI PIPLATFORM_TIME PIFInfo_getLastModified( PIFInfo *pFileInfo )
{
	if ( !pFileInfo )
		{ PIERROR( PIAPI_EINVAL ); return 0; };
	PICLEAR;
	return (PIPLATFORM_TIME)pFileInfo->GetLastModified();
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI const char *PIFInfo_getExtension( PIFInfo *pFileInfo )
{
	if ( !pFileInfo )
		{ PIERROR( PIAPI_EINVAL ); return 0; };
	return pFileInfo->GetExtension();
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI PIFInfo *PIFInfo_getFirstFileInDirectory( PIFInfo *pFileInfo )
{
	if ( !pFileInfo )
		{ PIERROR( PIAPI_EINVAL ); return 0; };
	return pFileInfo->FirstFileInDirectory( 0 );
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI int PIFInfo_getNextFileInDirectory( PIFInfo *pFileInfo )
{
	if ( !pFileInfo )
		{ PIERROR( PIAPI_EINVAL ); return 0; };
	return pFileInfo->NextSiblingFile() ?
		PIAPI_TRUE : PIAPI_FALSE;
}

