/*____________________________________________________________________________*\
 *

 Copyright (c) 1997 John Roy. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
    mentioning features must contain the following
    acknowledgment:
    "This product includes software developed by John Roy
    (http://www.johnroy.com/pi3/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: _PISync.cpp$
 * $Date: Sun Aug 10 06:29:42 1997$
 *
 Description:
\*____________________________________________________________________________*/
//$SourceTop:$

#include "OSEntity.h"
#include "PiAPI.h"
#include "Platform.h"
#include "PlatDefs.h"

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI int PISync_lock( Semaphore *pSync )
{
	if ( !pSync || !pSync->IsOK() )
		{ PIERROR( PIAPI_EINVAL ); return PIAPI_ERROR; };
	if ( pSync->Lock() )
		{ return PIAPI_ERROR; };
	return PIAPI_COMPLETED; 
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI int PISync_tryLock( Semaphore *pSync )
{
	if ( !pSync || !pSync->IsOK() )
		{ PIERROR( PIAPI_EINVAL ); return PIAPI_EINVAL; };
	int iRet = pSync->TryLock();
	if ( iRet == PISYNC_EBUSY )
		{ return PIAPI_WOULDBLOCK; };
	if ( iRet != 0 )
		{ return PIAPI_ERROR; };
	return PIAPI_COMPLETED; 
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI int PISync_unlock( Semaphore *pSync )
{
	if ( !pSync || !pSync->IsOK() )
		{ PIERROR( PIAPI_EINVAL ); return PIAPI_EINVAL; };
	if ( pSync->UnLock() )
		{ return PIAPI_ERROR; };
	return PIAPI_COMPLETED; 
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI int PISync_delete( Semaphore *pSync )
{
	if ( !pSync )
		{ PIERROR( PIAPI_EINVAL ); return PIAPI_ERROR; };
	PI_DELETE( pSync );
	return PIAPI_COMPLETED; 
}

